###
### Account Services Manager - Login
###
########################################################################
### This program is free software; you can redistribute it and/or
### modify it under the terms of the GNU General Public License
### as published by the Free Software Foundation; either version 2
### of the License, or (at your option) any later version.
###
### This program is distributed in the hope that it will be useful,
### but WITHOUT ANY WARRANTY; without even the implied warranty of
### MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
### GNU General Public License for more details.
###
### You should have received a copy of the GNU General Public License
### along with this program; if not, write to the Free Software
### Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
########################################################################

# Loading stuff
use vars qw(%Global_Config %HTTP_Args @HashElements);
require "../lib/main.pl";
API::LoadAuthModule();

# Startup stuff
my $redir = $HTTP_Args{'redir'} ? $HTTP_Args{'redir'} : ($ENV{'HTTP_REFERER'} ? $ENV{'HTTP_REFERER'}: '/');
my $user = lc($HTTP_Args{'Username'});
my @DenyUsers = split(/\s/, $Global_Config{'DenyUsers'}[1]);

# What strings will be used to generate a random hash for authentication?
@HashElements = (
    a .. z,
    A .. Z,
    0 .. 9,
    '.',
    '-',
    '+',
    '_',
    '|',
    '*',
    '!',
    '^',
    '&',
    '/',
    ','
);

# Authentication modules *SHOULD* verify the user exists, but just in case this one doesn't...
if($user)
{
    unless(-e "$BaseDir/users/$user" && -r "$BaseDir/users/$user")
    {
        PrintForm("Invalid username/password");
        exit;
    }
    my($hostname, $ipaddr) = GetHostRequireInfo();
    if(API::CheckPassword($user, $HTTP_Args{'Password'}))
    {
        foreach(@DenyUsers)
        {
            if($_ eq $user)
            {
                PrintForm("User account disabled by administator.");
                Log("Login denied for $user due to configuration.");
                exit;
            }
        }

        # Verify their remote location for this account.
        if(!VerifyHost($hostname, $ipaddr))
        {
            PrintForm("You are not allowed to login from this location.");
            Log("Host-based authentication failed for user $user.");
            exit;
        }

        # Get the authentication tokens.
        my $logintime = time();
        my $rnd_hash;
        for(my $i = 0; $i < $Global_Config{'RandomHashChars'}[1]; $i++)
        {
            $rnd_hash .= @HashElements[rand $#HashElements];
        }

        # Send the session state info to the client.
        print "Set-Cookie: asm=$user`:`$logintime`:`$rnd_hash; path=/\n";

        # Create their session.
        open(LOCK, ">$BaseDir/lock/$user.$logintime");
        print LOCK "$ENV{'REMOTE_ADDR'}\n";
        print LOCK "$ENV{'REMOTE_HOST'}\n";
        print LOCK "$rnd_hash";
        close LOCK;

        # Log this action.
        Log("$user logged in.");

        # Send a success page back to the client.
        print "REFRESH: 1;URL=$redir\n";
        print "Content-type: text/html\n\n";
        print qq~<html>
        <head><title>Login Successful</title></head>
        <body>
        <p><font face="Arial">Login successful! Click <a href=\"$redir\">here</a> to continue if your browser doesn't
        automatically forward you.</font></p>
        </body>
        </html>~;
    } else
    {
        Log("Invalid password entered for user $user");
        PrintForm("Invalid username/password");
    }
} else
{
    PrintForm();
}

sub GetHostRequireInfo
{
    my @buffer, @return;

    open(PASS, "$BaseDir/users/$user") || FatalError("The following error occured loading the user config file $BaseDir/users/$user: $!");
    my @lines = split(/\n/, join('', <PASS>));
    close PASS;

    foreach(@lines)
    {
        @buffer = split(/: /);
        $return[0] = $buffer[1] if($buffer[0] eq 'RequireHost');
        $return[1] = $buffer[1] if($buffer[0] eq 'RequireAddr');
    }
    return @return;
}

sub Log
{
    my $text = $_[0];
    my $time = localtime(time);

    open(LOG, ">>$BaseDir/logs/asm.log") || return;
    print LOG "[$time $ENV{'REMOTE_ADDR'} login] $text\n";
    close LOG;
}

sub VerifyHost
{
    my($hostname, $ipaddr) = @_;

    my $ip_buffer = $ipaddr;
    $ip_buffer =~ s/\./\\\./g;
    $ip_buffer =~ s/\*/./g;

    return 1 if($ENV{'REMOTE_ADDR'} =~ /^$ip_buffer/);

    my $host_buffer = $hostname;
    $host_buffer =~ s/\./\\\./g;
    $host_buffer =~ s/\*/./g;

    # If the hostname start with a '.' then allow ANYTHING before it to match.
    if(substr($host_buffer, 0, 1) eq '.')
    {
        return 1 if($ENV{'REMOTE_HOST'} =~ /^.$host_buffer$/);
    }
    else {
        return 1 if($ENV{'REMOTE_HOST'} =~ /^$host_buffer$/);
    }

    return 0;
}

sub PrintForm
{
    SendHeaders();
    my $error = $HTTP_Args{'error'};
    (!$error) and $error = $_[0];

    if($error) {
        $redir = '/login.cgi';
    }

    # Get the HTML theme for the login page.
    open(THEME, "$ENV{'DOCUMENT_ROOT'}/themes/$Global_Config{'CurrentTheme'}[1]/login.html");
    my $output = join('', <THEME>);
    close THEME;

    # Supply the theme with information it may need.
    my $version = GetVersion();
    $output =~ s/<!-- Registered Owner -->/$Global_Config{'RegisteredOwner'}[1]/gi;
    $output =~ s/<!-- Registered URL -->/$Global_Config{'RegisteredURL'}[1]/gi;
    $output =~ s/<!-- Username -->/$user/gi;
    $output =~ s/<!-- ASM Version -->/$version/gi;
    $output =~ s/<!-- Script_URI -->/$ENV{'SCRIPT_NAME'}/gi;
    $output =~ s/<!-- Redirect URL -->/$redir/gi;
    $output =~ s/<!-- Error Message -->/$error/gi;

    # Finish the page.
    print $output;
    print "\n</html>";
}
