###
### Main Library for the Account Services Manager
###
########################################################################
### This program is free software; you can redistribute it and/or
### modify it under the terms of the GNU General Public License
### as published by the Free Software Foundation; either version 2
### of the License, or (at your option) any later version.
###
### This program is distributed in the hope that it will be useful,
### but WITHOUT ANY WARRANTY; without even the implied warranty of
### MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
### GNU General Public License for more details.
###
### You should have received a copy of the GNU General Public License
### along with this program; if not, write to the Free Software
### Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
########################################################################

# Declare variables.
my $version = 0.8;
my $headers_sent = 0;
use vars qw($BaseDir %UserConf %scripts);

# Library initialization
GetConf();
$BaseDir = $Global_Config{'BaseDir'}[1];

# Only CGI's need to do these things.
if($ENV{'SCRIPT_NAME'} =~ /.cgi$/)
{
    LoadHTTPArgs();
    LoadAPI();
    ValidateConfig();
    CheckStatus();
    LoadScripts();
    LoadModuleList();
    do "$BaseDir/lib/html.pl";
    API::LoadSQL_Config();
}

# This function sets up the global configuration from asm.conf into %Global_Config.
sub GetConf
{
    # Get the contents of the config file.
    my $buffer, @tmp, $name, $value;
    open(CONF, "../conf/asm.conf") || FatalError("Cannot open the configuration file for reading: $!");
    $buffer = join('', <CONF>);
    $buffer = $1 if($buffer =~ /^([^<]*)$/);
    close CONF;

    # Now parse it line by line.
    my @conf = split(/\n\r?/, $buffer);
    foreach(@conf)
    {
        # Comments and blank lines should be ignored.
        # "Blank lines" includes lines that are only whitespace.
        $_ =~ s/\s*#.*//g;
        next if($_ =~ /^\s*\t*$/);

        @tmp = split(/\s/);
        $name = shift @tmp;
        $value = join(' ', @tmp);
        if(!exists $Global_Config{$name})
        {
            $Global_Config{$name}[0] = 0;
        }
        $Global_Config{$name}[0]++;
        $Global_Config{$name}[$Global_Config{$name}[0]] = $value;
    }
}

# Validate some of the core directives.
sub ValidateConfig
{
    if(!-d $Global_Config{'BaseDir'}[1]) {
        FatalError("Invalid base directory $Global_Config{'BaseDir'}[1]. Check for typos or permission errors.");
    }
    if($Global_Config{'BaseDir'}[0] > 1) {
        FatalError("There should only be 1 BaseDir directive in the configuration file.");
    }
    if(!exists $Global_Config{'Status'}) {
        FatalError("Invalid configuration; Status directive missing.");
    }
    if(!exists $Global_Config{'CurrentTheme'}) {
        API::LogError("No CurrentTheme directive in asm.conf. Assuming default.", 2);
        $Global_Config{'CurrentTheme'}[0] = 1;
        $Global_Config{'CurrentTheme'}[1] = 'gray';
    }
    if(!exists $Global_Config{'ModuleList'}) {
        $Global_Config{'ModuleList'}[0] = 1;
        $Global_Config{'ModuleList'}[1] = 'conf/modules.conf';
        if(-e $Global_Config{'ModuleList'}[1]) {
            API::LogError("No ModuleList directive was found. Assuming default.", 2);
        }
        else {
            FatalError("Unable to locate module configuration file. There is no entry in the configuration file pointing to the correct file.");
        }
    }
    if(!exists $Global_Config{'MinErrorLogLevel'}) {
        API::LogError("No MinErrorLogLevel directive in asm.conf. Assuming default.", 2);
        $Global_Config{'MinErrorLogLevel'}[0] = 1;
        $Global_Config{'MinErrorLogLevel'}[1] = 1;
    }
    if(!exists $Global_Config{'AuthModule'}) {
        API::LogError("No AuthModule directive found. Assuming default.", 2);
        $Global_Config{'AuthModule'}[0] = 1;
        $Global_Config{'AuthModule'}[1] = 'acctmgr';
    }
    if(!exists $Global_Config{'RandomHashChars'}) {
        API::LogError("No RandomHashChars directive found in asm.conf file. Assuming default.", 2);
        $Global_Config{'RandomHashChars'}[0] = 1;
        $Global_Config{'RandomHashChars'}[1] = 10;
    }
}

# Make sure the account manager hasn't been disabled.
sub CheckStatus
{
    if(lc($Global_Config{'Status'}[1]) eq 'offline') {
        SendHeaders();
        print "<html>\n";
        print "<head>\n";
        print "<title>Account Services Manager - Disabled</title>\n";
        print "</head>\n";
        print "<body bgcolor=\"#FFFFFF\" text=\"#000000\" vlink=\"#000000\">\n";
        print "<h1 align=\"center\"><u><font face=\"arial\">Account Services Manager</font></u></h1>\n";
        print "<p align=\"center\"><font face=\"arial\">Account Services Manager has been taken offline by an administrator.<br>\n";
        print "Contact <a href=\"mailto:$Global_Config{'RegisteredEmail'}[1]\">$Global_Config{'RegisteredOwner'}[1]</a> with any questions.</font></p>\n";
        print "</body>\n";
        print "</html>\n";
        exit;
    }
}

# Evaluate all the configuration variables.
sub EvalConfVars
{
    my $i;

    foreach $key(keys %Global_Config)
    {
        for($i = 1; $i <= $Global_Config{$key}[0]; $i++)
        {
            $Global_Config{$key}[$i] =~ s/\(Username\)/$AUTH::User/gi;
            foreach(keys %UserConf)
            {
                $Global_Config{$key}[$i] =~ s/\($_\)/$UserConf{'$_'}/gi;
            }
        }
    }

    ($Global_Config{'MinErrorLogLevel'}[1] eq "debug") and $Global_Config{'MinErrorLogLevel'}[1] = LOG_LEVEL_DEBUG;
    ($Global_Config{'MinErrorLogLevel'}[1] eq "info")  and $Global_Config{'MinErrorLogLevel'}[1] = LOG_LEVEL_INFO ;
    ($Global_Config{'MinErrorLogLevel'}[1] eq "warn")  and $Global_Config{'MinErrorLogLevel'}[1] = LOG_LEVEL_WARN ;
    ($Global_Config{'MinErrorLogLevel'}[1] eq "crit")  and $Global_Config{'MinErrorLogLevel'}[1] = LOG_LEVEL_CRIT ;
    ($Global_Config{'MinErrorLogLevel'}[1] eq "fatal") and $Global_Config{'MinErrorLogLevel'}[1] = LOG_LEVEL_FATAL;
}

# Send the HTTP Headers.
sub SendHeaders
{
    if(!$headers_sent)
    {
        printf("Application: Account Services Manager v%s\n", GetVersion());
        printf("Content-type: text/html\n\n");
        $headers_sent = 1;
    }
}

# Loads the HTTP arguments passed to the script into %HTTP_Args.
sub LoadHTTPArgs
{
    my($buffer, $name, $value, @args);
    if($ENV{'REQUEST_METHOD'} eq "POST") {
        read(STDIN, $buffer, $ENV{'CONTENT_LENGTH'});
    } else {
        $buffer = $ENV{'QUERY_STRING'};
    }
    @args = split(/&/, $buffer);
    foreach(@args)
    {
        ($name, $value) = split(/=/);
        $value =~ tr/+/ /;
        $value =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
        $value =~ /^([^<]*)$/;
        $value = $1;
        $name =~ tr/+/ /;
        $name =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
        $HTTP_Args{$name} = $value;
    }
}

# Load the hash of scripts names/URLs.
sub LoadScripts
{
    open(LIST, "$BaseDir/conf/scripts.conf");
    my @lines = split(/\n\r?/, join('', <LIST>));
    close LIST;

    my $name, $url;
    foreach(@lines)
    {
        next if($_ =~ /^#/ || $_ eq '');
        $_ = $1 if($_ =~ /^([^<]*)$/);
        ($name, $url) = split(/\t/);
        $scripts{$name} = $url;
    }
}

# Load all the installed modules.
sub LoadModuleList
{
    my $module_list = $Global_Config{'ModuleList'}[1];
    (!$module_list) and $module_list = 'conf/modules.conf';
    $module_list = $BaseDir . '/' . $module_list unless(substr($module_list, 0, 1) eq '/');

    open(LIST, $module_list) || FatalError("Could not load the modules list: $!");
    my @modules = split(/\n\r?/, join('', <LIST>));
    close LIST;

    my @DeniedModules = split(/\s/, $UserConf{'DenyModules'});
    my $DenyAll = 0;
    if($DeniedModules[0] eq '*')
    {
        $DenyAll = 1;
        shift @DeniedModules;
    }

    my $name, @data, $i;
    MODULE: foreach(@modules)
    {
        next if($_ =~ /^#*\s*\t*$/);
        ($name, @data) = split(/\t/);
        foreach $hidden(@DeniedModules) 
        {
            if(!$DenyAll){
                next MODULE if($hidden eq $name);
            }else{
                if($hidden eq $name){
                    last;
                }else{
                    next MODULE;
                }
            }
        }
        for($i = 0; $i <= $#data; $i++)
        {
            $data[$i] = $1 if($data[$i] =~ /^([^<]*)$/);
            $modules{$name}[$i] = $data[$i];
        }
    }
}

# Execute the module currently loaded.
sub ExecuteModule
{
    HTML::PrintTop();
    if(defined(&Main))
    {
        Main($#HTTP_Args, %HTTP_Args);
    }
    else
    {
        print "<font face=\"verdana, arial\" size=\"2\">Error: No Main() function defined in module '$module'. Contact your system administrator.</font>";
    }
    HTML::PrintBottom();
}

# Return the current version number for the whole program.
sub GetVersion
{
    return "$version";
}

# Load the ASM API library.
sub LoadAPI
{
    # Always load the main API library, that's the point of this function.
    eval{ require "../lib/api.pl" } || FatalError("Unable to load API library! Reason: $@");

    # Now load all the admin-defined libraries.
    my $i = 0;
    foreach(@Global_Config{'LoadLibrary'})
    {
        if($i != 0) {
            return 0 if(!API::LoadLibrary($_));
        }
        $i++;
    }

    return 1;
}

# Fatal error sub-routine.
sub FatalError
{
    SendHeaders();

    print "<h1 align=\"center\"><u>Account Services Manager</u></h1>\n";
    print "<p>An internal error has occured. $_[0]</p>\n";

    print "<p>Base directory: $BaseDir</p>\n";

    if($Global_Config{'DumpEnvOnError'}[1] eq 'True')
    {
        print "<h1><u>Enviroment Variables:</u></h1>\n";
        print "<p>\n";

        foreach(sort keys %ENV)
        {
            print "$_: $ENV{$_}<br>\n";
        }

        print "</p>\n";
    }

    if($Global_Config{'DumpConfigOnError'}[1] eq 'True')
    {
        print "<h1><u>Configuration:</u></h1>\n";

        my $i;
        foreach(sort keys %Global_Config)
        {
            for($i = 1; $i <= $Global_Config{$_}[0]; $i++) {
                print "$_ [$i]: $Global_Config{$_}[$i]<br>\n";
            }
        }
        print "</p>\n";
    }

    if($Global_Config{'DumpUserGroupOnError'}[1] eq 'True')
    {
        print "<h1><u>Current User & Group:</u></h1>\n";

        my @pw = getpwuid($<);
        print "Real User ID: $< ($pw[0])<br>\n";

        @pw = getpwuid($>);
        print "Effective User ID: $> ($pw[0])<br>\n";

        my @gr = getgrgid($();
        print "Real Group ID: $( ($gr[0])<br>\n";

        @gr = getgrgid($));
        print "Effective Group ID: $) ($gr[0])<br>\n";
    }

    print STDERR $_[0];
    exit;
}

1;
