###
### Account Services Manager - FTP module
###

# Set the display title.
$MODULE::Title = "FTP Accounts";

# How many FTP Accounts can we show on a page at a time?
my $MAX_ACCOUNTS_PER_PAGE = 5;

# Should we hide accounts not belonging to this user?
my $HIDE_OTHER_ACCOUNTS = 1;

# Find the uid & gid of the user.
my($uid, $gid) = (getpwnam($AUTH::User))[2, 3];

# Global variables
use vars qw(%accounts);
my $io_error = 0;

# Add an FTP account.
sub AddFTPAccount
{
    my $user = $HTTP_Args{'user'};
    my $pass = $HTTP_Args{'pass'};
    my $name = $HTTP_Args{'name'};
    my $home = $HTTP_Args{'home'};

    $home = "/$home" if($home);

    if(exists $UserConf{'MaxFTPs'} && $UserConf{'MaxFTPs'} ne 'x')
    {
        if($#accounts >= $::UserConf{'MaxFTPs'}){
            print "<font face=\"verdana, arial\" size=\"2\">Error: You've reached your maximum number of FTP accounts. Request denied.</font>\n";
            return 0;
        }
    }

    if(!$user)
    {
        # Print HTML form.
        print qq~<form method="post" action="$ENV{'SCRIPT_NAME'}">
<input type="hidden" name="module" value="ftp">
<input type="hidden" name="action" value="add">
<table border="0" cellpadding="1" cellspacing="0" align="center" width="500">
 <tr>
  <td colspan="2" align="center"><font size="+1">Add FTP Account</font></td>
 </tr>
 <tr>
  <td><font face="verdana, arial" size="2">Username:</font></td>
  <td><input type="text" name="user" value=""></td>
 </tr>
 <tr>
  <td><font face="verdana, arial" size="2">Password:</font></td>
  <td><input type="password" name="pass" value=""></td>
 </tr>
 <tr>
  <td><font face="verdana, arial" size="2">Real name (optional):</font></td>
  <td><input type="text" name="name" value=""></td>
 </tr>
 <tr>
  <td><font face="verdana, arial" size="2">Home directory:</font></td>
  <td><select name="home">
<option value="">Top-level access</option>
~;

    my @dirs = API::RecursiveList($::UserConf{'Home'});

    foreach(sort @dirs)
    {
        print "<option value=\"$_\">$_</option>\n";
    }
print qq~  </select></td>
 </tr>
 <tr>
  <td colspan="2" align="center"><input type="submit" value="Add User"></td>
 </tr>
</table>
</form>~;
        return 1;
    }

    return 0 if($io_error);

    if(exists $accounts{$user}){
        print "<font face=\"verdana, arial\" size=\"2\">Error: User $user already exists!</font>\n";
        return 0;
    }

    my $count = 0;
    foreach(split(//, $user))
    {
        $count++;
        if((!API::is_in_array($_, a..z))&&(!API::is_in_array($_, 0..9))&&($_ ne '.')&&($_ ne '_'))
        {
            print "<font face=\"verdana, arial\" size=\"2\">Error: Character #$count invalid: $_</font>\n";
            return 0;
        }
    }

    my $passwd = crypt($pass, API::RandomSalt());

    # Everything looks ok, so let's add the account already!
    open(AUTH, ">>$Global_Config{'FTPAuth'}[1]") || io_error(1);
    print AUTH "$user:$passwd:$uid:$gid:$name:$UserConf{'Home'}$home:\n";
    close AUTH;

    print "<font face=\"verdana, arial\" size=\"2\">$user successfully added to ftp.$UserConf{'Domain'}</font>\n";
    API::LogAction("$user (name: $name; home:$UserConf{'Home'}$home) added to ftp.$UserConf{'Domain'}");
    return 0;
}

# Update an account.
sub UpdateFTPAccount_Save
{
    my $user = $HTTP_Args{'user'};
    my $pass = $HTTP_Args{'pass'};
    my $name = $HTTP_Args{'name'};
    my $home = $HTTP_Args{'home'};
 
    $home = "/$home" if($home);
 
    if(!exists $accounts{$user}){
        print "<font face=\"verdana, arial\" size=\"2\">Error: User $user not found.</font>\n";
        return 0;
    }
    
    # In case they decide to get smart and try bypassing the - on the update column.
    if($accounts{$user}{'uid'} ne $uid)
    {
        print "<font face=\"verdana, arial\" size=\"2\">Error: You are not authorized to make changes to this account.</font>\n";
        return 0;
    }
    
    return 0 if($io_error);

    my $changes = 0;
    if($pass)
    {
        $accounts{$user}{'passwd'} = crypt($pass, API::RandomSalt());
        $changes++;
    }
    if($accounts{$user}{'home'} ne "$UserConf{'Home'}$home")
    {
        $accounts{$user}{'home'} = "$UserConf{'Home'}$home";
        $changes++;
    }
    if($name ne $accounts{$user}{'name'})
    {
        $accounts{$user}{'name'} = $name;
        $changes++;
    }

    if($changes)
    {
        UpdateAuthFile();
        printf("<font face=\"verdana, arial\" size=\"2\">User %s successfully updated. %d change(s) were(was) made.</font>\n", $user, $changes);
        API::LogAction("$changes made to $user on ftp.$UserConf{'Domain'}");
    }else{
        print "<font face=\"verdana, arial\" size=\"2\">No changes made because the submitted information is already current.</font>\n";
    }
}

sub UpdateFTPAccount
{
    my $user = $HTTP_Args{'user'};

    if(!exists $accounts{$user}){
        print "<font face=\"verdana, arial\" size=\"2\">Error: User $user not found.</font>\n";
        return 0;
    }
 
    # In case they decide to get smart and try bypassing the - on the update column.
    if($accounts{$user}{'uid'} ne $uid)
    { 
        print "<font face=\"verdana, arial\" size=\"2\">Error: You are not authorized to make changes to this account.</font>\n";
        return 0;
    }

    # Print HTML form.
    print qq~<form method="post" action="$ENV{'SCRIPT_NAME'}">
<input type="hidden" name="module" value="ftp">
<input type="hidden" name="action" value="updatesave">
<input type="hidden" name="user" value="$user">
<table border="0" cellpadding="1" cellspacing="0" align="center" width="500">
 <tr>
  <td colspan="2" align="center"><font size="+1">Update FTP Account</font></td>
 </tr>
 <tr>
  <td><font face="verdana, arial" size="2">Username:</font></td>
  <td><font face="verdana, arial" size="2">$user</font></td>
 </tr>  
 <tr>
  <td><font face="verdana, arial" size="2">New password:</font></td>
  <td><input type="password" name="pass" value=""></td>
 </tr>
 <tr>
  <td><font face="verdana, arial" size="2">Real name (optional):</font></td>
  <td><input type="text" name="name" value="$accounts{$user}{'name'}"></td>
 </tr>
 <tr>
  <td><font face="verdana, arial" size="2">Home directory:</font></td>
  <td><select name="home">
<option value="">Top-level access</option>
~;
 
    chdir($UserConf{'Home'});
    my $current_dir = '.';
    
    opendir(HOME, $current_dir);
    my @dirs = readdir(HOME);
    closedir HOME;
    @dirs = API::filterdirs($current_dir, @dirs);
 
    my @tmp_arr;
    foreach(@dirs)
    {
        $_ =~ s/^\.\///;
        $current_dir .= "/$_";
        opendir(SUBDIR, $current_dir);
        @tmp_arr = API::filterdirs($current_dir, readdir(SUBDIR));
        @dirs = (@dirs, @tmp_arr);
        closedir(SUBDIR);
        $current_dir = '.';
    }
    undef @tmp_arr;

    my $home = $accounts{$user}{'home'};
    $home =~ s/^$::UserConf{'Home'}\///i;

    foreach(sort @dirs)
    {
        if($_ eq $home){
            print "<option value=\"$_\" selected>$_</option>\n";
        }else{
            print "<option value=\"$_\">$_</option>\n";
        }
    }
    print qq~  </select></td>
 </tr>
 <tr>
  <td colspan="2" align="center"><input type="submit" value="Update User"></td>
 </tr>
</table>
</form>~;
    return 1;
}

# Delete an FTP account.
sub DeleteFTPAccount
{
    my $user      = shift;
    my $confirmed = $HTTP_Args{'confirmed'};

    if(!exists $accounts{$user}){
        print "<font face=\"verdana, arial\" size=\"2\">Error: User $user not found.</font>\n";
        return 0;
    }

    # In case they decide to get smart and try bypassing the - on the delete column.
    if($accounts{$user}{'uid'} ne $uid)
    {
        print "<font face=\"verdana, arial\" size=\"2\">Error: You are not authorized to delete this account.</font>\n";
        return 0;
    }

    if($user eq $AUTH::User)
    {
        print "<font face=\"verdana, arial\" size=\"2\">Error: Cannot delete the main webmaster user!</font>\n";
        return 0;
    }

    return 0 if($io_error);

    if($confirmed eq 'yes')
    {
        my $current_name = $accounts{$user}{'name'};    # Backup the name before we delete.
        delete $accounts{$user};    # Delete the FTP user.
        UpdateAuthFile($_);
        my $tmpuser = $user;
        $tmpuser .= " ($current_name)" if($current_name);
        print "<font face=\"verdana, arial\" size=\"2\">FTP user $tmpuser successfully deleted.</font>\n";
        API::LogAction("FTP user $tmpuser deleted from ftp.$UserConf{'Domain'}");
    }else{
        print qq~<form method="get" action="$ENV{'SCRIPT_NAME'}">
<input type="hidden" name="module" value="ftp">
<input type="hidden" name="action" value="delete">
<input type="hidden" name="user" value="$user">
<input type="hidden" name="confirmed" value="yes">
<font face="verdana, arial" size="2">Are you sure you want to delete user $user?</font><br>
<input type="submit" value="Yes"> &nbsp;<input type="button" value="No" onclick="window.href='index.cgi?module=ftp';">
</form>~;
        return 1;
    }

    return 0;
}

### Main Function ###
sub Main
{
    GetFTPAccounts();

    # Perform any desired actions before we display the main table.
    my $action = $HTTP_Args{'action'};
    if($action eq 'add'){
        if(AddFTPAccount())
        {
            PrintBackLink();
            return;
        }
    }elsif($action eq 'update'){
        if(UpdateFTPAccount())
        {
            PrintBackLink();
            return;
        }
    }elsif($action eq 'updatesave'){
        UpdateFTPAccount_Save();
    }elsif($action eq 'delete'){
        return if(DeleteFTPAccount($HTTP_Args{'user'}));
    }

    GetFTPAccounts();    # Refresh the list.

    print "<table border=\"1\" cellpadding=\"2\" cellspacing=\"1\" align=\"center\" width=\"90%\">\n";
    print " <tr>\n  <td align=\"center\" colspan=\"5\"><font face=\"verdana, arial\" size=\"4\"><u><b>FTP Accounts</b></u></font></td>\n </tr>\n";
    print " <tr>\n";
    print "  <td><font face=\"verdana, arial\" size=\"2\"><u>Username</u></font></td>\n";
    print "  <td><font face=\"verdana, arial\" size=\"2\"><u>Real name</u></font></td>\n";
    print "  <td><font face=\"verdana, arial\" size=\"2\"><u>Home directory</u></font></td>\n";
    print "  <td><font face=\"verdana, arial\" size=\"2\"><u>Update</u></font></td>\n";
    print "  <td><font face=\"verdana, arial\" size=\"2\"><u>Delete</u></font></td>\n";
    print " </tr>\n";
    my $NumberOfAccounts = 0;
    my $NumberShown = 0;
    my $offset = 0;
    $offset = $HTTP_Args{'offset'} if(exists $HTTP_Args{'offset'});
    foreach(sort keys %accounts)
    {
        if($NumberOfAccounts >= $offset && $NumberOfAccounts < $offset + $MAX_ACCOUNTS_PER_PAGE){
            print " <tr>\n";
            print "  <td><font face=\"verdana, arial\" size=\"2\">$_</font></td>\n";
            if($accounts{$_}{'name'}){
                print "  <td><font face=\"verdana, arial\" size=\"2\">$accounts{$_}{'name'}</font></td>\n";
            }else{
                print "  <td align=\"center\"><font face=\"verdana, arial\" size=\"2\">[Not Set]</font></td>\n";
            }
            my $home = $accounts{$_}{'home'};
            $home =~ s/^$UserConf{'Home'}//i;
            if($home){
                print "  <td><font face=\"verdana, arial\" size=\"2\">$home</font></td>\n";
            }else{
                print "  <td align=\"center\"><font face=\"verdana, arial\" size=\"2\">[Top-level access]</font></td>\n";
            }
            if($accounts{$_}{'uid'} eq $uid) {
                print "  <td align=\"center\"><a href=\"index.cgi?module=ftp&action=update&user=$_\"><img src=\"/themes/$Global_Config{'CurrentTheme'}[1]/button.gif\" border=\"0\"></a></td>\n";
                if($_ eq $AUTH::User){
                    print "  <td align=\"center\"><font face=\"verdana, arial\" size=\"2\">-</font></td>\n";
                }else{
                    print "  <td align=\"center\"><a href=\"index.cgi?module=ftp&action=delete&user=$_\"><img src=\"/themes/$Global_Config{'CurrentTheme'}[1]/button.gif\" border=\"0\"></a></font></td>\n";
                }
            }else{
                print "  <td align=\"center\"><font face=\"verdana, arial\" size=\"2\">-</font></td>\n" x 2;
            }
            print " </tr>\n";
            $NumberShown++;
        }
        $NumberOfAccounts++;
    }
    if($NumberOfAccounts)
    {
        my $num1 = $offset + 1;
        my $num2 = $offset + $MAX_ACCOUNTS_PER_PAGE;
        $num2    = $num2 - ($num2 - $NumberOfAccounts) if($num2 > $NumberOfAccounts);
        print "  <td colspan=\"5\" align=\"center\"><font face=\"verdana, arial\" size=\"2\">$NumberOfAccounts account(s) found, $num1 through $num2 displayed.</font></td>\n";
        print " </tr>\n";
        print " <tr>\n";
        my $previous = $offset - $MAX_ACCOUNTS_PER_PAGE;
        my $next     = $offset + $MAX_ACCOUNTS_PER_PAGE;
        print "  <td colspan=\"5\">\n";
        print "<table border=\"0\" cellpadding=\"0\" cellspacing=\"0\" width=\"100%\">\n";
        print " <tr>\n";
        my $prev, $forw;
        if($previous < 0){
            $prev = "<font color=\"gray\">&lt;&lt;Previous</font>";
        }else{
            $prev = "<a href=\"index.cgi?module=ftp&offset=$previous\">&lt;&lt;Previous</a>";
        }
        print "  <td align=\"center\" width=\"50%\"><font face=\"verdana, arial\" size=\"2\">$prev</font></td>\n";
        if($next >= $NumberOfAccounts){
            $forw = "<font color=\"gray\">Next>></font>";
        }else{
            $forw = "<a href=\"index.cgi?module=ftp&offset=$next\">Next>></a>";
        }
        print "  <td align=\"center\"><font face=\"verdana, arial\" size=\"2\">$forw</font></td>\n";
        print " </tr>\n";
        print "</table>\n";
        print "  </td>\n";
    }else
    {
        print "  <td colspan=\"5\" align=\"center\"><font face=\"verdana, arial\" size=\"2\">No ftp accounts are setup on ftp.$UserConf{'Domain'}</font></td>\n";
    }
    print " </tr>\n";
    print "</table>\n";

    print "<p align=\"center\">\n";
    if($io_error)
    {
        print "<font face=\"verdana, arial\" size=\"2\">ERROR: An I/O error occured in this module. Description: $!</font>\n";
    }else
    {
        print "<font face=\"verdana, arial\" size=\"2\"><b>Options:</b> <a href=\"index.cgi?module=ftp&action=add\">Add FTP Account</a> \| <a href=\"index.cgi?module=ftp&offset=$offset\">Refresh List</a> \| <a href=\"index.cgi?module=ftp\">Beginning of List</a></font>\n";
    }
    print "</p>\n";
}

# Get a list of FTP accounts for this account.
sub GetFTPAccounts
{
    %accounts = ();
    my($user, $passwd, $uid, $gid, $name, $home, $shell);
    open(PASSWD, $Global_Config{'FTPAuth'}[1]) || io_error(1);
    while(<PASSWD>)
    {
        next if($_ =~ /^#/);
        ($user, $passwd, $uid, $gid, $name, $home, $shell) = split(/:/);
        if(!($uid != (getpwnam($AUTH::User))[2] && $HIDE_OTHER_ACCOUNTS != 0))
        {
            $accounts{$user} = {
                passwd => $passwd,
                uid    => $uid,
                gid    => $gid,
                name   => $name,
                home   => $home,
                shell  => $shell
            };
        }
    }
    close PASSWD;
}

# Update the authentication file.
sub UpdateAuthFile
{
    my $user = shift;
    my($passwd, $name, $home);
    open(PASSWD, ">$Global_Config{'FTPAuth'}[1]") || io_error(1);
    foreach(keys %accounts)
    {
        my($passwd, $uid, $gid, $name, $home) = ($accounts{$_}{'passwd'}, $accounts{$_}{'uid'}, $accounts{$_}{'gid'}, $accounts{$_}{'name'}, $accounts{$_}{'home'});
        print PASSWD "$_:$passwd:$uid:$gid:$name:$home:\n";
    }
    close PASSWD;
}

# If an I/O error occured, modifying the list is pretty useless.
sub io_error
{
    $io_error = $_[0];
}

# Print a link to go back to the main page.
sub PrintBackLink
{
    print "<p align=\"center\"><font face=\"verdana, arial\" size=\"2\"><a href=\"index.cgi?module=ftp\">Back to main list...</a></font></p>\n";
}

1;
