###
### Mail Admin Module v1.0
###

###
### This module is designed to be compatible with QmailAdmin.
### Everything the user does here should be seen by QmailAdmin as well.
###

package QMAIL;

# Set the title for this module.
$MODULE::Title = "Mail Admin";

# We need some javascript functions to be loaded along with the module.
$MODULE::Header = qq~<script language="JavaScript" type="text/javascript">
<!--
/* Check the password and confirm password for POP3 Admin. */
function CheckPasswords()
{
    if(document.addpop3.password.value == document.addpop3.confirmpassword.value)
        return true;
    else{
        window.alert("Error: Your password and confirm password do not match!");
        return false;
    }
}

/* Confirm the deletion of a mailbox. */
function DeleteMailbox(user)
{
    if(confirm("Are you sure you want to delete " + user + "\@$::UserConf{'Domain'}?"))
        window.location.href = "index.cgi?module=qmail&opt=pop3&action=del&user=" + user;
}
//-->
</script>~;

# Global variables
my $MailDir, %accounts, $vpopmail_dir;

# What's the maximum number of accounts to display per page?
my $MAX_ACCOUNTS_PER_PAGE = 7;

# Where is the autorespond program?
my $AUTORESPOND_DIR = $::Global_Config{'AutorespondDir'}[1];

# Main Function
sub ::Main
{
    my($argc, %args) = @_;
    my $uid = $<;
    my @vpopmail = getpwnam($::Global_Config{'VpopmailUser'}[1]);
    $vpopmail_dir = $vpopmail[7];
    $MailDir = "$vpopmail_dir/domains/$::UserConf{'Domain'}";

    # Their virtual domain's folder has to exist or this module is pretty much useless...
    if(!-e $MailDir || !-d $MailDir){
        print "<font face=\"verdana, arial\" size=\"2\">FATAL ERROR: There is no mail server for $::UserConf{'Domain'}</font>";
        return;
    }

    GetAccounts();

    if($::HTTP_Args{'opt'} eq 'pop3') {
        print DoPOP3($::HTTP_Args{'action'});
    }elsif($::HTTP_Args{'opt'} eq 'aliases') {
        print DoAliases($::HTTP_Args{'action'});
    }elsif($::HTTP_Args{'opt'} eq 'forwarders') {
        print DoForwarders($::HTTP_Args{'action'});
    }elsif($::HTTP_Args{'opt'} eq 'catchall') {
        print DoCatchAll($::HTTP_Args{'action'});
    }elsif($::HTTP_Args{'opt'} eq 'autoresponders') {
        print DoAutoresponders($::HTTP_Args{'action'});
    }else{
        ShowMenu();
    }
}

# Manage POP3's
sub DoPOP3
{
    my $action = shift;
    my $return_html = "<h2 align=\"center\"><font face=\"verdana, arial\"><u>POP3 Administration</u></font></h2>\n";

    if($action eq 'add'){
        open(POP3, "$MODULE::Path/addpop3.html");
        my $form = join('', <POP3>);
        $form =~ s/--script_name--/$::ENV{'SCRIPT_NAME'}/gi;
        close POP3;
        return $form;
    }
    elsif($action eq 'addsave'){
        my $user = $::HTTP_Args{'name'};
        my $pass = $::HTTP_Args{'password'};
        my $errorout = 0;

        if(exists $accounts{$user}){
            $return_html = "Error: $user\@$::UserConf{'Domain'} already exists!";
            $errorout = 1;
        }
        unless($::UserConf{'MaxPOP3s'} eq 'x'){
            if(CountAccounts('pop3') >= $::UserConf{'MaxPOP3s'}){
                $return_html = "Error: Maximum number of mailboxes reached!";
                $errorout = 1;
            }
        }
        if($user =~ /@/){
            $return_html = "Error: Invalid name!";
            $errorout = 1;
        }
        if(!$errorout){
            if(!system("$vpopmail_dir/bin/vadduser", "$user\@$::UserConf{'Domain'}", $pass)){
                ModuleLog("POP3 $user\@$::UserConf{'Domain'} added.");
                $return_html = "POP3 Mailbox $user\@$::UserConf{'Domain'} successfully added!";
            }else{
                $return_html = "An internal error occured adding mailbox $user. Contact your system administrator.";
            }
        }
    }
    elsif($action eq 'mod'){
        open(POP3, "$::BaseDir/modules/qmail/modpop3.html");
        my $form = join('', <POP3>);
        $form =~ s/--script_name--/$::ENV{'SCRIPT_NAME'}/gi;
        $form =~ s/--username--/$::HTTP_Args{'user'}/gi;
        close POP3;
        return $form;
    }
    elsif($action eq 'modsave'){
        my $username = $::HTTP_Args{'user'};
        my $password = $::HTTP_Args{'password'};
        my $errorout = 0;
        if(!exists $accounts{$username} || $accounts{$username} ne 'pop3'){
            $return_html = "Error: No such mailbox $username.";
            $errorout = 1;
        }
        if(!$errorout){
            if(!system("cd $vpopmail_dir/bin && ./vpasswd $username\@$::UserConf{'Domain'} \"$password\"")){
                ModuleLog("POP3 $username\@$::UserConf{'Domain'} updated.");
                $return_html = "POP3 Mailbox successfully updated!";
            }else{
                $return_html = "An error occured modifying the POP3. Please try again. If the problem persists, contact your system administrator.";
            }
        }
    }
    elsif($action eq 'del'){
        my $errorout = 0;
        my $user     = $::HTTP_Args{'user'};
        if(!exists $accounts{$user}){
            $return_html = "Error: No such mailbox!";
            $errorout = 1;
        }
        if($user eq 'postmaster'){
            $return_html = "Error: Postmaster is a special account! You cannot delete it.";
            $errorout = 1;
        }
        if($user eq GetCatchAll()){
            $return_html = "Error: Cannot delete $user because that account is set as the catchall.";
            $errorout = 1;
        }
        if(!$errorout){
            if(!system("cd $vpopmail_dir/bin && ./vdeluser $user\@$::UserConf{'Domain'}")){
                ModuleLog("POP3 $user\@$::UserConf{'Domain'} deleted.");
                $return_html = "POP3 Mailbox $user\@$::UserConf{'Domain'} successfully deleted!";
            }else{
                $return_html = "An internal error occured deleting mailbox $user. Contact your system administrator.";
            }
        }
    }

    # Refresh the list of accounts.
    GetAccounts();

    $return_html .=  "<table border=\"0\" cellpadding=\"1\" cellspacing=\"2\" align=\"center\" width=\"500\">\n";
    $return_html .= " <tr>\n  <td><font face=\"verdana, arial\" size=\"2\"><u>Name</u></font></td>\n  <td><font face=\"verdana, arial\" size=\"2\"><u>Modify</u></font></td>\n  <td><font face=\"verdana, arial\" size=\"2\"><u>Delete</u></font></td>\n </tr>\n";

    my $NumberOfAccounts = 0;
    my $NumberShown = 0;
    my $offset = 0;
    $offset = $::HTTP_Args{'offset'} if(exists $::HTTP_Args{'offset'});
    foreach(sort keys %accounts){
        if($accounts{$_}[0] eq 'pop3'){
            if($NumberOfAccounts >= $offset && $NumberOfAccounts < $offset + $MAX_ACCOUNTS_PER_PAGE){
                $return_html .= " <tr>\n";
                $return_html .= "  <td><font face=\"verdana, arial\" size=\"2\">$_</font></td>\n";
                $return_html .= "  <td><font face=\"verdana, arial\" size=\"2\"><a href=\"index.cgi?module=qmail&opt=pop3&action=mod&user=$_\"><img src=\"/themes/$::Global_Config{'CurrentTheme'}[1]/button.gif\" border=\"0\"></a></font></td>\n";
                if($_ eq 'postmaster' || $_ eq GetCatchAll()){
                    $return_html .= "  <td><font face=\"verdana, arial\" size=\"2\">-</font></td>\n";
                }else{
                    $return_html .= "  <td><font face=\"verdana, arial\" size=\"2\"><a href=\"javascript:DeleteMailbox('$_');\"><img src=\"/themes/$::Global_Config{'CurrentTheme'}[1]/button.gif\" border=\"0\"></a></font></td>\n";
                }
                $return_html .= " </tr>\n";
                $NumberShown++;
            }
            $NumberOfAccounts++;
        }
    }
    if($NumberShown)
    {
        my $num1  = $offset + 1;
        my $num2  = $offset + $MAX_ACCOUNTS_PER_PAGE;
        $num2     -= $num2 - $NumberOfAccounts if($num2 > $NumberOfAccounts);
        $return_html .= " <tr><td colspan=\"3\" align=\"center\"><font face=\"verdana, arial\" size=\"2\">$NumberOfAccounts mailboxes found, displaying $num1 to $num2</font></td></tr>\n";
        $return_html .= " </tr>\n";
        $return_html .= " <tr>\n";
        my $previous = $offset - $MAX_ACCOUNTS_PER_PAGE;
        my $next     = $offset + $MAX_ACCOUNTS_PER_PAGE;
        $return_html .= "  <td colspan=\"5\" align=\"center\">\n";
        $return_html .= "<table border=\"0\" cellpadding=\"0\" cellspacing=\"0\" width=\"100%\">\n";
        $return_html .= " <tr>\n";
        my $prev, $forw;
        if($previous < 0){
            $prev = "<font color=\"gray\">&lt;&lt;Previous</font>";
        }else{
            $prev = "<a href=\"index.cgi?module=qmail&opt=pop3&offset=$previous\">&lt;&lt;Previous</a>";
        }
        $return_html .= "  <td><font face=\"verdana, arial\" size=\"2\">$prev</font></td>\n";
        if($next >= $NumberOfAccounts){
            $forw = "<font color=\"gray\">Next>></font>";
        }else{
            $forw = "<a href=\"index.cgi?module=qmail&opt=pop3&offset=$next\">Next>></a>";
        }
        $return_html .= "  <td><font face=\"verdana, arial\" size=\"2\">$forw</font></td>\n";
        $return_html .= " </tr>\n";
        $return_html .= "</table>\n";
        $return_html .= "  </td>\n";
        $return_html .= " </tr>\n";
    }else{
        $return_html .= " <tr><td colspan=\"3\" align=\"center\"><font face=\"verdana, arial\" size=\"2\">No POP3 mailboxes setup for $::UserConf{'Domain'}.</font></td></tr>\n";
    }
    $return_html .= "</table>\n";

    $return_html .= "<p align='center'><a href='index.cgi?module=qmail&opt=pop3&action=add'>Add POP3</a> \| <a href='index.cgi?module=qmail'>Main Menu</a> \| <a href='index.cgi?module=qmail&opt=pop3'>Refresh List</a></p>\n";

    return $return_html;
}

# Handle Forwarders
sub DoForwarders
{
    my $action = shift;
    my $return_html;

    if($action eq 'add'){
        open(FOR, "$::BaseDir/modules/qmail/addforwarder.html");
        my $form = join('', <FOR>);
        $form =~ s/--script_name--/$::ENV{'SCRIPT_NAME'}/gi;
        close FOR;
        return $form;
    }elsif($action eq 'addsave'){
        my $localname = $::HTTP_Args{'name'};
        my $forwardto = $::HTTP_Args{'forwardto'};
        my $errorout  = 0;
        unless($::UserConf{'MaxForwarders'} eq 'x'){
            if(CountAccounts('forwarder') >= $::UserConf{'MaxForwarders'}){
                $return_html = "Error: Maximum number of forwarders reached!";
                $errorout = 1;
            }
        }
        if($localname =~ /\@/ || $localname =~ /\s/){
            $return_html = "Invalid name!";
            $errorout = 1;
        }
        if($localname eq 'default'){
            $return_html = "Error: For internal reasons you cannot have a forwarder or alias called default!";
            $errorout = 1;
        }
        if(exists $accounts{$localname}){
            $return_html = "Error: $localname\@$::UserConf{'Domain'} already exists! It is a<font size=\"-1\">(n)</font> $accounts{$localname}[0].";
            $errorout = 1;
        }
        if(!$errorout){
            open(QMAIL, ">$MailDir/.qmail-$localname") || print "The following error occured creating the .qmail file: $!";
            print QMAIL "&$forwardto";
            close QMAIL;
            ModuleLog("Forwarder $localname\@$::UserConf{'Domain'} added. Pointing to $forwardto");
            $return_html = "Forwarder $localname\@$::UserConf{'Domain'} successfully created!";
        }
    }elsif($action eq 'del'){
        if(-e "$MailDir/.qmail-$::HTTP_Args{'name'}"){
            if(unlink("$MailDir/.qmail-$::HTTP_Args{'name'}")){
                ModuleLog("Forwarder $::HTTP_Args{'name'}\@$::UserConf{'Domain'} deleted.");
                $return_html = "Forwarder/alias removed successfully.";
            }else{
                $return_html = "The following error occured: $!";
            }
        }else{
            $return_html = "No such name '$::HTTP_Args{'name'}'";
        }
    }

    # Refresh the account list.
    GetAccounts();

    # Build an HTML Table with a list of forwarders.
    $return_html .=  "<table border=\"0\" cellpadding=\"1\" cellspacing=\"2\" align=\"center\" bordercolor=\"#FFFFFF\" width=\"400\">\n";
    $return_html .= " <tr>\n";
    $return_html .= "  <td><font face=\"verdana, arial\" size=\"2\"><u>Name</u></font></td>\n";
    $return_html .= "  <td><font face=\"verdana, arial\" size=\"2\"><u>Delete</u></font></td>\n";
    $return_html .= "  <td><font face=\"verdana, arial\" size=\"2\"><u>Forwards to</u></font></td>\n";
    $return_html .= " </tr>\n";

    my $NumberOfAccounts = 0;
    my $NumberShown = 0;
    my $offset = 0;
    $offset = $::HTTP_Args{'offset'} if(exists $::HTTP_Args{'offset'});
    foreach(sort keys %accounts){
        if($accounts{$_}[0] eq 'forwarder'){
            if($NumberOfAccounts >= $offset && $NumberOfAccounts < $offset + $MAX_ACCOUNTS_PER_PAGE){
                $return_html .= " <tr>\n";
                $return_html .= "  <td><font face=\"verdana, arial\" size=\"2\">$_</font></td>\n";
                $return_html .= "  <td><font face=\"verdana, arial\" size=\"2\"><a href=\"index.cgi?module=qmail&opt=forwarders&action=del&name=$_\"><img src=\"/themes/$::Global_Config{'CurrentTheme'}[1]/button.gif\" border=\"0\"></a></font></td>\n";
                $return_html .= "  <td><font face=\"verdana, arial\" size=\"2\">$accounts{$_}[1]</font></td>\n";
                $return_html .= " </tr>\n";
                $NumberShown++;
            }
            $NumberOfAccounts++;
        }
    }

    if($NumberShown)
    {
        my $num1  = $offset + 1;
        my $num2  = $offset + $MAX_ACCOUNTS_PER_PAGE;
        $num2     -= $num2 - $NumberOfAccounts if($num2 > $NumberOfAccounts);
        $return_html .= " <tr><td colspan=\"3\" align=\"center\"><font face=\"verdana, arial\" size=\"2\">$NumberOfAccounts $::HTTP_Args{'opt'} found, displaying $num1 to $num2</font></td></tr>\n";
        $return_html .= " </tr>\n";
        $return_html .= " <tr>\n";
        my $previous = $offset - $MAX_ACCOUNTS_PER_PAGE;
        my $next     = $offset + $MAX_ACCOUNTS_PER_PAGE;
        $return_html .= "  <td colspan=\"5\" align=\"center\">\n";
        $return_html .= "<table border=\"0\" cellpadding=\"0\" cellspacing=\"0\" width=\"100%\">\n";
        $return_html .= " <tr>\n";
        my $prev, $forw;
        if($previous < 0){
            $prev = "<font color=\"gray\">&lt;&lt;Previous</font>";
        }else{
            $prev = "<a href=\"index.cgi?module=qmail&opt=$::HTTP_Args{'opt'}&offset=$previous\">&lt;&lt;Previous</a>";
        }
        $return_html .= "  <td><font face=\"verdana, arial\" size=\"2\">$prev</font></td>\n";
        if($next >= $NumberOfAccounts){
            $forw = "<font color=\"gray\">Next>></font>";
        }else{
            $forw = "<a href=\"index.cgi?module=qmail&opt=$::HTTP_Args{'opt'}&offset=$next\">Next>></a>";
        }
        $return_html .= "  <td><font face=\"verdana, arial\" size=\"2\">$forw</font></td>\n";
        $return_html .= " </tr>\n";
        $return_html .= "</table>\n";
        $return_html .= "  </td>\n";
        $return_html .= " </tr>\n";
    }else{
        $return_html .= " <tr><td colspan=\"3\" align=\"center\"><font face=\"verdana, arial\" size=\"2\">No $::HTTP_Args{'opt'} setup for $::UserConf{'Domain'}.</font></td></tr>\n";
    }
    $return_html .= "</table>\n";

    $return_html .= "<p align='center'><a href='index.cgi?module=qmail&opt=forwarders&action=add'>Add Forwarder</a> \| <a href='index.cgi?module=qmail'>Main Menu</a> \| <a href='index.cgi?module=qmail&opt=forwarders'>Refresh List</a></p>\n";

    return $return_html;
}

# Handle Aliases
sub DoAliases
{
    my $action = shift;

    my $return_html;

    if($action eq 'add') {
        my $list = '';
        foreach(keys %accounts) {
            if($accounts{$_}[0] eq 'pop3') {
                $list .= " <option value=\"$_\">$_</option>\n";
            }
        }
        open(FOR, "$::BaseDir/modules/qmail/addalias.html");
        my $form = join('', <FOR>);
        $form =~ s/--script_name--/$::ENV{'SCRIPT_NAME'}/gi;
        $form =~ s/--pop3-list--/$list/gi;
        close FOR;
        return $form;
    }elsif($action eq 'addsave') {
        my $localname = $::HTTP_Args{'name'};
        my $alias     = $::HTTP_Args{'alias'};
        my $errorout  = 0;
        if($alias =~ /@/) {
           $return_html = "Error: Invalid e-mail address!";
            $errorout = 1;
        }
        $alias = $::HTTP_Args{'alias'} . '@' . $::UserConf{'Domain'};
        unless($::UserConf{'MaxAliases'} eq 'x') {
            if(CountAccounts('alias') >= $::UserConf{'MaxAliases'}) {
                $return_html = "Error: Maximum number of aliases reached!";
                $errorout = 1;
            }
        }
        if($localname =~ /\@/ || $localname =~ /\s/){
            $return_html = "Invalid name!";
            $errorout = 1;
        }
        if($localname eq 'default'){
            $return_html = "Error: For internal reasons you cannot have a forwarder or alias called default!";
            $errorout = 1;
        }
        if(exists $accounts{$localname}){
            $return_html = "Error: $localname\@$::UserConf{'Domain'} already exists! It is a<font size=\"-1\">(n)</font> $accounts{$localname}[0].";
            $errorout = 1;
        }
        if(!$errorout){
            open(QMAIL, ">$MailDir/.qmail-$localname") || print "The following error occured creating the .qmail file: $!";
            print QMAIL "$MailDir/$::HTTP_Args{'alias'}/Maildir/\n";
            close QMAIL;
            ModuleLog("Alias $localname\@$::UserConf{'Domain'} added. Pointing to $alias");
            $return_html = "Alias $localname\@$::UserConf{'Domain'} successfully created!";
        }
    }elsif($action eq 'del') {
        if(-e "$MailDir/.qmail-$::HTTP_Args{'name'}"){
            if(unlink("$MailDir/.qmail-$::HTTP_Args{'name'}")){
                ModuleLog("Alias $::HTTP_Args{'name'}\@$::UserConf{'Domain'} deleted.");
                $return_html = "Alias removed successfully.";
            }else{
                $return_html = "The following error occured: $!";
            }
        }else{
            $return_html = "No such name '$::HTTP_Args{'name'}'";
        }
    }

    # Refresh the account list.
    GetAccounts();

    # Build an HTML Table with a list of aliases.
    $return_html .=  "<table border=\"0\" cellpadding=\"1\" cellspacing=\"2\" align=\"center\" bordercolor=\"#FFFFFF\" width=\"400\">\n";
    $return_html .= " <tr>\n";
    $return_html .= "  <td><font face=\"verdana, arial\" size=\"2\"><u>Name</u></font></td>\n";
    $return_html .= "  <td><font face=\"verdana, arial\" size=\"2\"><u>Delete</u></font></td>\n";
    $return_html .= "  <td><font face=\"verdana, arial\" size=\"2\"><u>Alias To</u></font></td>\n";
    $return_html .= " </tr>\n";

    my $NumberOfAccounts = 0;
    my $NumberShown = 0;
    my $offset = 0;
    $offset = $::HTTP_Args{'offset'} if(exists $::HTTP_Args{'offset'});

    foreach(sort keys %accounts){
        if($accounts{$_}[0] eq 'alias'){
            if($NumberOfAccounts >= $offset && $NumberOfAccounts < $offset + $MAX_ACCOUNTS_PER_PAGE){
                $return_html .= " <tr>\n";
                $return_html .= "  <td><font face=\"verdana, arial\" size=\"2\">$_</font></td>\n";
                $return_html .= "  <td><font face=\"verdana, arial\" size=\"2\"><a href=\"index.cgi?module=qmail&opt=aliases&action=del&name=$_\"><img src=\"/themes/$::Global_Config{'CurrentTheme'}[1]/button.gif\" border=\"0\"></a></font></td>\n";
                $return_html .= "  <td><font face=\"verdana, arial\" size=\"2\">$accounts{$_}[1]</font></td>\n";
                $return_html .= " </tr>\n";
                $NumberShown++;
            }          
            $NumberOfAccounts++;
        }
    }

    if($NumberShown)
    {
        my $num1  = $offset + 1;
        my $num2  = $offset + $MAX_ACCOUNTS_PER_PAGE;
        $num2     -= $num2 - $NumberOfAccounts if($num2 > $NumberOfAccounts);
        $return_html .= " <tr><td colspan=\"3\" align=\"center\"><font face=\"verdana, arial\" size=\"2\">$NumberOfAccounts $::HTTP_Args{'opt'} found, displaying $num1 to $num2</font></td></tr>\n";
        $return_html .= " </tr>\n";
        $return_html .= " <tr>\n";
        my $previous = $offset - $MAX_ACCOUNTS_PER_PAGE;
        my $next     = $offset + $MAX_ACCOUNTS_PER_PAGE;
        $return_html .= "  <td colspan=\"5\" align=\"center\">\n";
        $return_html .= "<table border=\"0\" cellpadding=\"0\" cellspacing=\"0\" width=\"100%\">\n";
        $return_html .= " <tr>\n";
        my $prev, $forw;
        if($previous < 0){
            $prev = "<font color=\"gray\">&lt;&lt;Previous</font>";
        }else{
            $prev = "<a href=\"index.cgi?module=qmail&opt=$::HTTP_Args{'opt'}&offset=$previous\">&lt;&lt;Previous</a>";
        }
        $return_html .= "  <td><font face=\"verdana, arial\" size=\"2\">$prev</font></td>\n";
        if($next >= $NumberOfAccounts){
            $forw = "<font color=\"gray\">Next>></font>";
        }else{
            $forw = "<a href=\"index.cgi?module=qmail&opt=$::HTTP_Args{'opt'}&offset=$next\">Next>></a>";
        }
        $return_html .= "  <td><font face=\"verdana, arial\" size=\"2\">$forw</font></td>\n";
        $return_html .= " </tr>\n";
        $return_html .= "</table>\n";
        $return_html .= "  </td>\n";
        $return_html .= " </tr>\n";
    }else{
        $return_html .= " <tr><td colspan=\"3\" align=\"center\"><font face=\"verdana, arial\" size=\"2\">No $::HTTP_Args{'opt'} setup for $::UserConf{'Domain'}.</font></td></tr>\n";
    }
    $return_html .= "</table>\n";

    $return_html .= "<p align='center'><a href='index.cgi?module=qmail&opt=aliases&action=add'>Add Alias</a> \| <a href='index.cgi?module=qmail'>Main Menu</a> \| <a href='index.cgi?module=qmail&opt=aliases'>Refresh List</a></p>\n";

    return $return_html;
}

# Handle the Catch-all account.
sub DoCatchAll
{
    my $action = shift;
    my $return_html = "<h2 align=\"center\"><font face=\"verdana, arial\"><u>Catch-All Account</u></font></h2>\n";
    $return_html .= qq~<font face="verdana, arial" size="2">
    A catch-all account is a local pop3 mailbox that receives all bounced messages. Let's say for example that someone sends
    an e-mail to webmaster\@$::UserConf{'Domain'} and you have not setup webmaster\@$::UserConf{'Domain'} yet. Normally
    the message will be bounced back to the sender. But with Account Services Manager, you can define a local pop3 mailbox that the incoming
    message will be forwarded to rather than bouncing it. This section let's you configure this option. Disabling catch-all will
    cause the mail server to bounce all incoming messages if the mailbox doesn't exist.</font>~;

    if($action eq 'disable'){
        my $disable = DisableCatchAll();
        if($disable){
            $return_html .= $disable;
        }else{
            ModuleLog("Catch-all disabled for $::UserConf{'Domain'}.");
            $return_html .= "<font face=\"verdana, arial\" size=\"2\">Catch-all successfully disabled.</font>";
        }
    }
    elsif($action eq 'set'){
        my $user = $::HTTP_Args{'user'};
        my $errorout = 0;
        if(!exists $accounts{$user} || $accounts{$user}[0] ne 'pop3'){
            $return_html .= "<font face=\"verdana, arial\" size=\"2\">Error: No such mailbox '$user'!</font>";
            $errorout = 1;
        }
        if($user eq GetCatchAll()){
            $return_html .= "<font face=\"verdana, arial\" size=\"2\">Error: Your catch-all account is already set to $user\@$::UserConf{'Domain'}</font>";
            $errorout = 1;      # This *could* prevent some useless disk writing, assuming the OS doesn't buffer it...
        }
        if(!$errorout){
            open(DEF, ">$MailDir/.qmail-default") || return $return_html . "<br><br>Error: Could not open .qmail-default: $!";
            print DEF "| $vpopmail_dir/bin/vdelivermail '' $MailDir/$user";
            close DEF || return $return_html . "<br><br>Internal error occured saving .qmail-default: $!";
            ModuleLog("Catch-all for $::UserConf{'Domain'} set to $user.");
            $return_html .= "<font face=\"verdana, arial\" size=\"2\">Catch-all account successfully set to $user\@$::UserConf{'Domain'}</font>";
        }
    }
    else{
        $return_html .= qq~<p align="center">\n<form method="post" action="index.cgi">
        <input type="hidden" name="module" value="qmail">
        <input type="hidden" name="opt" value="catchall">
        <input type="hidden" name="action" value="set">~;
        $return_html .= "\n <font face=\"verdana, arial\" size=\"2\">Set the catch-all to:&nbsp;</font>\n <select name=\"user\">\n";
        foreach(sort keys %accounts){
            if($accounts{$_}[0] eq 'pop3'){
                if($_ eq GetCatchAll()){
                    $return_html .= "  <option value=\"$_\" selected>$_</option>\n";
                }else{
                    $return_html .= "  <option value=\"$_\">$_</option>\n";
                }
            }
        }
        $return_html .= " </select>\n";
        $return_html .= " &nbsp;<input type=\"submit\" value=\"Set Catch-All Account\">\n";
        $return_html .= " </form></p>\n";
        if(GetCatchAll() ne ""){
            $return_html .= " <p align=\"center\"><font face=\"verdana, arial\" size=\"2\"><a href=\"index.cgi?module=qmail&opt=catchall&action=disable\">Disable Catch-All</a></font></p>\n";
        }else{
            $return_html .= " <p align=\"center\"><font face=\"verdana, arial\" size=\"2\">Catch-all disabled. Select a user above to set the catch-all.</font>\n";
        }
    }

    return $return_html;
}

# Handle autoresponders
sub DoAutoresponders
{
    my $return_html  = "<h2 align=\"center\"><font face=\"verdana, arial\"><u>Autoresponders</u></font></h2>\n";
    my $action       = shift;
    my $name         = $::HTTP_Args{'name'};
    my $forward_addr = $::HTTP_Args{'forward_addr'};
    my $subject      = $::HTTP_Args{'subject'};
    my $message      = $::HTTP_Args{'message'};
    my $mime;
    if(exists $::HTTP_Args{'HTML'})
    {
        $mime = "text/html";
    }else{
        $mime = "text/plain";
    }

    if($action eq 'add'){
        # If a field was missing, show them the form.
        if(!$name || !$subject || !$message)
        {
            open(AUTO, "$::BaseDir/modules/qmail/addautoresponder.html");
            my $form = join('', <AUTO>);
            $form =~ s/--script_name--/$::ENV{'SCRIPT_NAME'}/gi;
            close AUTO;
            return $form;
        }

        # Calculate a directory name for the autoresponder message. It's compatible with QmailAdmin.
        my $add_dir = "$MailDir/" . uc($name);

        my $errorout = 0;

        if(!$name){
            $return_html .= "<font face=\"verdana, arial\" size=\"2\">Error: You must specify a name for this autoresponder!</font>\n";
            $errorout = 1;
        }

        unless($::UserConf{'MaxAutoresponders'} eq 'x'){
            if(CountAccounts('autoresponder') >= $::UserConf{'MaxAutoresponders'}){
                $return_html .= "<font face=\"verdana, arial\" size=\"2\">Error: You've reached or exceeded your limit and cannot add any more autoresponders.</font>\n";
                $errorout = 1;
            }
        }

        if(exists $accounts{$name}){
            $return_html .= "<font face=\"verdana, arial\" size=\"2\">Error: $name\@$::UserConf{'Domain'} already exists!</font>\n";
            $errorout = 1;
        }

        if($name eq 'default'){
            $return_html .= "<font face=\"verdana, arial\" size=\"2\">Error: For internal reasons, you cannot have an autoresponder named default.</font>\n";
            $errorout = 1;
        }

        # Everything checks out, go ahead and add the autoresponder.
        if(!$errorout){
            # Create the autoresponder's directory.
            mkdir($add_dir, 0754);

            # Create the message file.
            open(MESSAGE, ">$add_dir/message");
            print MESSAGE "Content-type: $mime\n";
            print MESSAGE "From: $name\@$::UserConf{'Domain'}\n";
            print MESSAGE "Subject: $subject\n\n";
            print MESSAGE $message;
            close MESSAGE;

            # Create the .qmail file (the actual autoresponder).
            open(QMAIL, ">$MailDir/.qmail-$name");
            print QMAIL "\|$AUTORESPOND_DIR/autorespond 10000 5 $add_dir/message $add_dir";
            print QMAIL "\n&$forward_addr" if($forward_addr);
            close QMAIL;

            # Mission successful!
            ModuleLog("Autoresponder $name\@$::UserConf{'Domain'} added.");
            $return_html .= "<font face=\"verdana, arial\" size=\"2\">Autoresponder $name\@$::UserConf{'Domain'} successfully added.</font>";
        }
    }elsif($action eq 'mod'){
        # XXX: Some of this code has just been copy & pasted from the add routine, with a couple changes.

        # Calculate a directory name for the autoresponder message. It's compatible with QmailAdmin.
        my $add_dir = "$MailDir/" . uc($name);
                
        my $errorout = 0;
                    
        if(!$name){
            $errorout = 1;
        }

        if((!exists $accounts{$name} || $accounts{$name} ne 'autoresponder') && !$errorout){
            $return_html .= "<font face=\"verdana, arial\" size=\"2\">Error: Autoresponder $name\@$::UserConf{'Domain'} not found!</font>\n";
            $errorout = 1;
        }

        # Get the information.
        my %current;
        unless($errorout)
        {
            my $headers_ended = 0;
            open(MESSAGE, "$MailDir/" . uc($name) . "/message") || print "Couldn't open $MailDir/" . uc($name) . "/message: $!\n";
            while(<MESSAGE>)    # I hate doing things this way, but it's the only way I can think to handle this... *sigh*
            {
                unless($headers_ended)
                {
                    if($_ =~ /^Content-type: /)
                    {
                        $current{'mime'} = (split(/: /))[1];
                    }elsif($_ =~ /^From: /)
                    {
                        # Don't do anything with the From: header because that's dynamically calculated.
                        # This condition is here to prevent future headers from being seen as message text.
                    }elsif($_ =~ /^Subject: /)
                    {
                        $current{'subject'} = (split(/: /))[1];
                    }elsif($_ eq "\n")
                    {
                        $headers_ended = 1;    # We've reached the end of the headers, but don't add this blank line to the message.
                    }
                }else{
                    $current{'message'} .= $_;
                }
            }
            close MESSAGE;
            open(QMAIL, "$MailDir/.qmail-$name");
            $current{'forward_addr'} = (<QMAIL>)[1];
            $current{'forward_addr'} =~ s/^&//;
            close QMAIL;
        }
        foreach(keys %current)
        {
            $current{$_} =~ s/\n$//;
        }

        if((!$name || !$subject || !$message) && !$errorout)
        {
            open(AUTO, "$::BaseDir/modules/qmail/modautoresponder.html");
            my $form = join('', <AUTO>);
            $form =~ s/--script_name--/$::ENV{'SCRIPT_NAME'}/gi;
            if($current{'mime'} eq 'text/html'){
                $form =~ s/--use_html_checked--/ checked/gi;
            }else{
                $form =~ s/--use_html_checked--//gi;
            }
            $form =~ s/--subject--/$current{'subject'}/gi;
            $form =~ s/--message--/$current{'message'}/gi;
            $form =~ s/--forward_addr--/$current{'forward_addr'}/gi;
            $form =~ s/--name--/$name/gi;
            $form =~ s/--domain--/$::UserConf{'Domain'}/gi;
            close AUTO;
            return $form;
        }

        # Everything checks out, go ahead and update the autoresponder.
        if(!$errorout){
            # Update the message file.
            unless($mime eq $current{'mime'} && $subject ne $current{'subject'} && $message ne $current{'message'})
            {
                open(MESSAGE, ">$add_dir/message");
                print MESSAGE "Content-type: $mime\n";
                print MESSAGE "From: $name\@$::UserConf{'Domain'}\n";
                print MESSAGE "Subject: $subject\n\n";
                print MESSAGE $message;
                close MESSAGE;
            }

            # Update the .qmail file if nessecary.
            unless($forward_addr eq $current{'forward_addr'})
            {
                open(QMAIL, ">$MailDir/.qmail-$name");
                print QMAIL "\|$AUTORESPOND_DIR/autorespond 10000 5 $add_dir/message $add_dir";
                print QMAIL "\n&$forward_addr" if($forward_addr);
                close QMAIL;
            }

            # Mission successful!
            ModuleLog("Autoresponder $name\@$::UserConf{'Domain'} modified.");
            $return_html .= "<font face=\"verdana, arial\" size=\"2\">Autoresponder $name\@$::UserConf{'Domain'} successfully updated.</font>";
        }
    }elsif($action eq 'del'){
        my $errorout = 0;
        if(!exists $accounts{$name} || $accounts{$name} ne 'autoresponder'){
            $return_html .= "font face=\"verdana, arial\" size=\"2\">Error: autoresponder $name\@$::UserConf{'Domain'} not found.</font>\n";
            $errorout = 1;
        }
        unless($errorout){
            if(unlink("$MailDir/.qmail-$name") && !system("/bin/rm -Rf $MailDir/" . uc($name))){
                ModuleLog("Autoresponder $name\@$::UserConf{'Domain'} deleted.");
                $return_html .= "<font face=\"verdana, arial\" size=\"2\">Autoresponder $name\@$::UserConf{'Domain'} deleted successfully.</font>";
            }else{
                $return_html .= "<font face=\"verdana, arial\" size=\"2\">An internal error occured deleting the nessecary files.</font>\n";
                $return_html .= "<br><font face=\"verdana, arial\" size=\"2\">Last runtime error: $!</font>\n" if($!);
            }
        }
    }

    # Refresh the list of accounts.
    GetAccounts();

    $return_html .=  "<table border=\"0\" cellpadding=\"1\" cellspacing=\"2\" align=\"center\" bordercolor=\"#FFFFFF\" width=\"400\">\n";
    $return_html .= " <tr>\n  <td width=\"*\"><font face=\"verdana, arial\" size=\"2\"><u>Name</u></font></td>\n  <td width=\"20\"><font face=\"verdana, arial\" size=\"2\"><u>Modify</u></font></td>\n  <td width=\"20\"><font face=\"verdana, arial\" size=\"2\"><u>Delete</u></font></td>\n </tr>\n";

    my $NumberOfAccounts = 0;
    my $NumberShown = 0;
    my $offset = 0;
    $offset = $::HTTP_Args{'offset'} if(exists $::HTTP_Args{'offset'});
    foreach(sort keys %accounts){
        if($accounts{$_}[0] eq 'autoresponder'){
            if($NumberOfAccounts >= $offset && $NumberOfAccounts < $offset + $MAX_ACCOUNTS_PER_PAGE){
                $return_html .= " <tr>\n";
                $return_html .= "  <td><font face=\"verdana, arial\" size=\"2\">$_</font></td>\n";
                $return_html .= "  <td><font face=\"verdana, arial\" size=\"2\"><a href=\"index.cgi?module=qmail&opt=autoresponders&action=mod&name=$_\"><img src=\"/themes/$::Global_Config{'CurrentTheme'}[1]/button.gif\" border=\"0\"></a></font></td>\n";
                $return_html .= "  <td><font face=\"verdana, arial\" size=\"2\"><a href=\"index.cgi?module=qmail&opt=autoresponders&action=del&name=$_\"><img src=\"/themes/$::Global_Config{'CurrentTheme'}[1]/button.gif\" border=\"0\"></a></font></td>\n";
                $return_html .= " </tr>\n";
                $NumberShown++;
            }
            $NumberOfAccounts++;
        }
    }

    if($NumberShown)
    {
        my $num1  = $offset + 1;
        my $num2  = $offset + $MAX_ACCOUNTS_PER_PAGE;
        $num2     -= $num2 - $NumberOfAccounts if($num2 > $NumberOfAccounts);
        $return_html .= " <tr><td colspan=\"3\" align=\"center\"><font face=\"verdana, arial\" size=\"2\">$NumberOfAccounts autoresponders found, displaying $num1 to $num2</font></td></tr>\n";
        $return_html .= " </tr>\n";
        $return_html .= " <tr>\n";
        my $previous = $offset - $MAX_ACCOUNTS_PER_PAGE;
        my $next     = $offset + $MAX_ACCOUNTS_PER_PAGE;
        $return_html .= "  <td colspan=\"5\" align=\"center\">\n";
        $return_html .= "<table border=\"0\" cellpadding=\"0\" cellspacing=\"0\" width=\"100%\">\n";
        $return_html .= " <tr>\n";
        my $prev, $forw;
        if($previous < 0){
            $prev = "<font color=\"gray\">&lt;&lt;Previous</font>";
        }else{
            $prev = "<a href=\"index.cgi?module=qmail&opt=autoresponders&offset=$previous\">&lt;&lt;Previous</a>";
        }
        $return_html .= "  <td><font face=\"verdana, arial\" size=\"2\">$prev</font></td>\n";
        if($next >= $NumberOfAccounts){
            $forw = "<font color=\"gray\">Next>></font>";
        }else{
            $forw = "<a href=\"index.cgi?module=qmail&opt=autoresponders&offset=$next\">Next>></a>";
        }
        $return_html .= "  <td><font face=\"verdana, arial\" size=\"2\">$forw</font></td>\n";
        $return_html .= " </tr>\n";
        $return_html .= "</table>\n";
        $return_html .= "  </td>\n";
        $return_html .= " </tr>\n";
    }else{
        $return_html .= " <tr><td colspan=\"3\" align=\"center\"><font face=\"verdana, arial\" size=\"2\">No autoresponders setup for $::UserConf{'Domain'}.</font></td></tr>\n";
    }
    $return_html .= "</table>\n";

    $return_html .= "<p align='center'><a href='index.cgi?module=qmail&opt=autoresponders&action=add'>Add Autoresponder</a> \| <a href='index.cgi?module=qmail'>Main Menu</a> \| <a href='index.cgi?module=qmail&opt=autoresponders'>Refresh List</a></p>\n";

    return $return_html;
}

# Which account is the catch-all? Returns an empty string if none.
sub GetCatchAll
{
    open(DEF, "$MailDir/.qmail-default") || return "<font face=\"verdana, arial\" size=\"2\">Error: Could not open .qmail-default: $!. Contact your system administrator.";
    my $line = <DEF>;
    $line =~ s,^\| $vpopmail_dir/bin/vdelivermail '' $MailDir/,,i;
    $line =~ s,\n$,,;
    close DEF;

    ($line =~ /bounce-no-mailbox/) and return "";

    return $line;
}

# Disable catch-all
sub DisableCatchAll
{
    (GetCatchAll() eq "") and return "<font face=\"verdana, arial\" size=\"2\">Error: Your catchall is already disabled. Aborting operation...</font>";

    open(DEF, ">$MailDir/.qmail-default") || return "<font face=\"verdana, arial\" size=\"2\">Error: Could not open .qmail-default: $!. Contact your system administrator!</font>";
    print DEF "\| $vpopmail_dir/bin/vdelivermail '' bounce-no-mailbox";
    close DEF;

    return "";
}

# Make %accounts into a list of accounts for this user.
# account_name => 'account_type'
sub GetAccounts
{
    # Empty %accounts so that this function can be called to refresh, not just load.
    %accounts = ();

    # Get the POP3's
    my $pop3;
    open(VPASSWD, "$MailDir/vpasswd");
    while(<VPASSWD>){
        $pop3 = (split(/:/))[0];
        $accounts{$pop3}[0] = 'pop3';
    }
    close VPASSWD;
    undef $pop3;

    # Move on to the forwarders & aliases.
    opendir(MAILDIR, $MailDir);
    my @files = readdir(MAILDIR);
    closedir MAILDIR;

    for(my $i=0; $i<=$#files; $i++){
        if($files[$i] !~ /^.qmail-/ || $files[$i] eq '.' || $files[$i] eq '..' || -d "$MailDir/$files[$i]"){
            $files[$i] = '';
        }
    }
    my $line;
    foreach(@files){
        next if($_ eq '.qmail-default');
        next if($_ eq '');

        open(MAIL, "$MailDir/$_");
        $line = <MAIL>;
        close MAIL;

        $name = substr($_, 7, length($_) - 1);

        if($line =~ /\/autorespond/){
            $accounts{$name}[0] = 'autoresponder';
            $accounts{$name}[1] = '';
        }elsif($line =~ /$MailDir\/(.*)\/Maildir/){
            $accounts{$name}[0] = 'alias';
            $accounts{$name}[1] = $1;
        }elsif($line =~ /&(.*)/){
            $accounts{$name}[0] = 'forwarder';
            $accounts{$name}[1] = $1;
        }
    }
}

# How many accounts of a particular type are there?
sub CountAccounts
{
    my $type = shift;

    # Return the total number of accounts if no type was given.
    (!$type) and return $#accounts;

    my $count = 0;
    foreach(keys %accounts)
    {
        $count++ if($accounts{$_}[0] eq $type);
    }

    if($type eq 'pop3') {
        $count--;
    }

    return $count;
}

# Display a menu of all the qmail options.
sub ShowMenu
{
    print <<"MENU";
<h1 align="center"><u>Mail Administration</u></h1>
<p align="center"><font face="verdana, arial">
<a href="index.cgi?module=qmail&opt=pop3">POP3 Mailboxes</a><br>
<a href="index.cgi?module=qmail&opt=aliases">Aliases</a><br>
<a href="index.cgi?module=qmail&opt=forwarders">Forwarders</a><br>
<a href="index.cgi?module=qmail&opt=catchall">Catch-All Account</a><br>
<a href="index.cgi?module=qmail&opt=autoresponders">Autoresponders</a><br>
</font></p>
MENU
}

# Local logging routine for this module.
sub ModuleLog
{
    my $time = localtime;
    my $text = shift;

    open(LOG, ">>$MODULE::Path/log") || return 0;
    print LOG "[$time $ENV{'REMOTE_ADDR'} $AUTH::User] $::module: $text\n";
    return close LOG;
}

1;
