#!/usr/bin/perl

###
### Subdomain Reloader
### ==================
### By            : Brandon Ramirez
### Date          : Saturday, March 10, 2001
### Last Modified : Wednesday, February 13, 2002
### Email         : brandonramirez@users.sourceforge.net
###

###
###                                          The database format is as follows:
### ==================================================================================================================================================
### 0		1		2		3		4		5		6		7		8		9
### ServerName	VirtualHost	Username	DocumentRoot	ContactEmail	ServerAlias	ScriptAlias	AccessLog	ErrorLog	Extras
###

###
### Configuration
###
my $MINUTES_TO_SLEEP = 5;
my $LOG_FILE         = '/usr/local/asm/logs/srd.log';
my $MAIN_FILE        = '/usr/local/apache/conf/main.conf';
my $CONFIG_FILE      = '/usr/local/apache/conf/httpd.conf';

###
### Load everything we need.
###
$home = '/usr/local/asm';
require "$home/lib/main.pl";
LoadAPI();
API::LoadSQL_Config();
do "$home/modules/subdomains/subdomains.conf";
my $db_server = API::GetSQLHostname();
my $db_user   = API::GetSQLUsername();
my $db_pass   = API::GetSQLPassword();

use DBI;
$db_name = 'virtualhosts';
my($main_buffer, $date, $db, $query, @row, @directives, @extras, $main_last_mod, $main_last_mod_fs, $table_last_mod, $table_last_mod_db);

###
### Put this program into the background.
###
if($ARGV[0] ne '--nodaemon')
{
    print "Starting Subdomain Reloader...\n";
    fork() && exit(0);
    open(STDIN,  "/dev/null");
    open(STDOUT, ">$LOG_FILE");
    open(STDERR, ">>$LOG_FILE");
}

###
### Program Loop
###
while(1)
{
    ###
    ### Load the cached data from our last time through this loop.
    ###
    open(CACHE, "$home/modules/subdomains/cache");
    $main_last_mod = <CACHE>;
    chop $main_last_mod;
    $table_last_mod = <CACHE>;
    if($table_last_mod =~ /\n$/) {
        chop $table_last_mod;
    }
    close CACHE;

    ###
    ### Get the main configuration text.
    ###
    $main_last_mod_fs = (stat($MAIN_FILE))[9];
    if($main_last_mod_fs > $main_last_mod || $main_buffer eq '')
    {
        open(MAIN, $MAIN_FILE) || die "Error: Unable to open main configuration file: $!";
        $main_buffer = join('', <MAIN>);
        close MAIN;
    }

    ###
    ### Get the current time & date.
    ###
    $date = localtime;

    ###
    ### Start everything up...
    ###
    $db = DBI->connect("DBI:mysql:$db_name:$db_server", $db_user, $db_pass) or die "Error: Unable to connect to database $db_name. Reason: $!";
    $query = $db->prepare('show table status;');
    $query->execute;
    while(@row = $query->fetchrow_array)
    {
        if($row[0] eq $db_table)
        {
            $table_last_mod_db = $row[11];
        }
    }

    ###
    ### The big part: Rewrite the config file if either the main config file OR the database have been updated.
    ###
    if($main_last_mod_fs > $main_last_mod || $table_last_mod_db ne $table_last_mod)
    {
        open(FILE, ">$CONFIG_FILE");
        print FILE $main_buffer;
        print FILE "\n";
        print FILE "###\n";
        print FILE "### VirtualHost configuration section dynamically generated by the Account Services Manager on $date\n";
        print FILE "###\n";
        print FILE "\n";

        ###
        ### Get the database contents.
        ###
        $query = $db->prepare("select * from $db_table");
        $query->execute;

        ###
        ### Write each VirtualHost out into the configuration file.
        ###
        while(@directives = $query->fetchrow_array)
        {
            if($directives[10] != 1) {
                print FILE "<VirtualHost $directives[1]>\n";
                print FILE "ServerName $directives[0]\n";
                print FILE "DocumentRoot $directives[3]\n";
                if($directives[4]) {
                    print FILE "ServerAdmin $directives[4]\n";
                }
                if($directives[5]) {
                    print FILE "ServerAlias $directives[5]\n";
                }
                if($directives[6]) {
                    print FILE "ScriptAlias $directives[6]\n";
                }
                if($directives[7]) {
                    print FILE "CustomLog $directives[7] combined\n";
                }
                if($directives[8]) {
                    print FILE "ErrorLog $directives[8]\n";
                }
                @extras = split(/;/, $directives[9]);
                foreach(@extras)
                {
                    print FILE "$_\n";
                }
                print FILE "</VirtualHost>\n";
                print FILE "\n";
            }
        }
        close FILE;

        ###
        ### Reload the HTTP server so it'll recognize the new configuration.
        ###
        system('/usr/bin/killall', '-USR1', 'httpd');

        ###
        ### Update the cache.
        ###
        open(CACHE, ">$home/modules/subdomains/cache");
        print CACHE "$main_last_mod_fs\n";
        print CACHE "$table_last_mod_db\n";
        close CACHE;
    }

    ###
    ### Looks like we're done here, let's close out.
    ###
    $db->disconnect;

    ###
    ### Take a little break =)
    ###
    if($ARGV[0] ne '--nodaemon')
    {
        sleep($MINUTES_TO_SLEEP * 60);
    }
    else
    {
        exit(0);
    }
}
