###
### Account Services Manager - Subdomain Module
###

# Set the object name for this module. Not required, just makes life easier...
package SUBS;

# Set the title for this module:
$MODULE::Title = "Subdomains";

# We need some CSS to be loaded too.
$MODULE::Header = qq~<style type="text/css">
<!--
.inputbox {font-size: 10px}
-->
</style>~;

# Let's load the DBI module so we can actually access the subdomain config.
use DBI;

# Get the maximum number of subdomains allowed for this user (unlimited if no limit set).
my $MaxSubs;
if(exists $::UserConf{'MaxSubs'}) {
    $MaxSubs = 'x';
} else {
    $MaxSubs = $::UserConf{'MaxSubs'};
}

# Load Configuration
do "$::BaseDir/conf/sql.auth";
do "$MODULE::Path/subdomains.conf";

# Declare module-wide variables.
my($db, $NumberOfSubs);

# Add a subdomain to the user's account.
sub AddSubdomain
{
    my $subdomain    = lc(shift);
    my $virtualhost  = "$subdomain.$::UserConf{'Domain'}";
    my $server_admin = $::HTTP_Args{'ServerAdmin'};
    my $doc_root     = $::UserConf{'Home'} . '/' . $::HTTP_Args{'DocumentRoot'};

    if(!$subdomain)
    {
        print qq~<form method="post" action="$::ENV{'SCRIPT_NAME'}">
<input type="hidden" name="module" value="subdomains">
<input type="hidden" name="action" value="add">
<table border="0" cellpadding="2" cellspacing="2" align="center">
 <tr>
  <td colspan="2" align="center"><h3><font face="verdana, arial">Add Subdomain</font></h3></td>
 </tr>
 <tr>
  <td><font face="verdana, arial" size="2">Name:</font></td>
  <td><input type="text" name="name" value="" size="20" maxchars="20" class="inputbox">.<font face="verdana, arial" size="2">$::UserConf{'Domain'}</font></td>
 </tr>
 <tr>
  <td><font face="verdana, arial" size="2">Document Root:</font></td>
  <td><select name="DocumentRoot" class="inputbox">~;

    chdir($::UserConf{'Home'});
    my $current_dir = '.';

    opendir(HOME, $current_dir);
    my @dirs = readdir(HOME);
    closedir HOME;
    @dirs = API::filterdirs($current_dir, @dirs);

    my @tmp_arr;
    foreach(@dirs)
    {
        $_ =~ s/^\.\///;
        $current_dir .= "/$_";
        opendir(SUBDIR, $current_dir);
        @tmp_arr = API::filterdirs($current_dir, readdir(SUBDIR));
        @dirs = (@dirs, @tmp_arr);
        closedir(SUBDIR);
        $current_dir = '.';
    }
    undef @tmp_arr;

    foreach(sort @dirs)
    {
        print "<option value=\"$_\">$_</option>\n";
    }

print qq~</select></td>
 </tr>
 <tr>
  <td><font face="verdana, arial" size="2">Contact E-mail:</font></td>
  <td><input type="text" name="ServerAdmin" value="postmaster\@$::UserConf{'Domain'}" size="30" class="inputbox"></td>
 </tr>
 <tr>
  <td align="center" colspan="2"><input type="submit" value="Submit"><font size="-1"><br><a href="index.cgi?module=subdomains&action=makedir&oldaction=add">Make a new directory...</a></font></td>
 </tr>
</table>
</form>~;
        return 1;
    }

    # Check for invalid characters.
    foreach(split(//, $subdomain))
    {
        if((!API::is_in_array($_, a..z))&&(!API::is_in_array($_, 0..9))&&($_ ne '.')){
            print "<font face=\"verdana, arial\" size=\"2\">Error: '$subdomain' is not a valid subdomain name. Please use only alphanumeric characters or a .</font>\n";
            return 0;
        }
    }

    # We can't let them add a subdomain 5 million characters long, now can we?
    if(length($subdomain) > 20)    # 20 should be more than enough.
    {
        print "<font face=\"verdana, arial\" size=\"2\">Error: Subdomain name too long.</font>\n";
        return 0;
    }

    # Does the document root exist?
    if(!-e $doc_root || !-d $doc_root)
    {
        print "<font face=\"verdana, arial\" size=\"2\">Error: Document root not found!</font>\n";
        return 0;
    }

    # Make sure they don't exceed the max!
    if($MaxSubs ne 'x' && $NumberOfSubs >= $MaxSubs)
    {
        print "<font face=\"verdana, arial\" size=\"2\">Error: Your request to add $virtualhost has been declined because you have reached your limit.</font>\n";
        return 0;
    }

    my @row;
    my $list = $db->prepare("select * from virtualhosts");
    $list->execute;
    SUBDOM: while(@row = $list->fetchrow_array)
    {
        if($row[1] eq $AUTH::User)
        {
            next SUBDOM if($row[9] eq $::UserConf{'Domain'});    # Skip the main virtualhost, that's not a subdomain.
            if($row[5] eq $virtualhost)
            {
                print "<font face=\"verdana, arial\" size=\"2\">Error: Subdomain '$virtualhost' already exists!</font>\n";
                return 0;
            }
        }
    }    

    my $ip_addr = ($::UserAuth{'IPVhosting'} eq 'True') ? $::UserAuth{'IP'} : $virtualhost;
    my $add = $db->prepare("insert into httpd(ServerName,VirtualHost,Username,DocumentRoot,ContactEmail,AccessLog,ErrorLog) values('$virtualhost','$ip_addr','$AUTH::User','$doc_root','$server_admin','$custom_log','$error_log');");
    $add->execute;
    $NumberOfSubs++;

    API::LogAction("$virtualhost added; DocumentRoot: $doc_root");

    print "<font face=\"verdana, arial\" size=\"2\">Subdomain $virtualhost successfully added. Changes may take up to 10 minutes to activate.</font>\n";

    return 0;
}

# Delete a subdomain.
sub DeleteSubdomain
{
    my $virtualhost = (shift) . ".$::UserConf{'Domain'}";

    my $query = $db->prepare("delete from httpd where ServerName = '$virtualhost'");
    $query->execute;
    $NumberOfsubs--;

    API::LogAction("$virtualhost deleted.");

    print "<font face=\"verdana, arial\" size=\"2\">$virtualhost deleted successfully. Changes may take up to 10 minutes to activate.</font>\n";
}

# Modify a subdomain
sub ModifySubdomain
{
    my $subdomain = shift;
    my $virtualhost = "$subdomain.$::UserConf{'Domain'}";
    my $doc_root = $::HTTP_Args{'DocumentRoot'};
    my $server_admin = $::HTTP_Args{'ServerAdmin'};

    if(!$subdomain)
    {
        return 0;
    }

    my $list = $db->prepare("select * from virtualhosts");
    $list->execute;
    my @row, %current;
    while(@row = $list->fetchrow_array)
    {
        if($row[0] eq $virtualhost)
        {
            next if($row[5] eq $::UserConf{'Domain'});    # Skip the main virtualhost, that's not a subdomain.
            $current{'DocumentRoot'} = $row[3];
            $current{'ServerAdmin'}  = $row[4];
            $current{'DocumentRoot'} =~ s/^$::UserConf{'Home'}\///i;
            last;
        }
    }
    undef $list, @row;

    if(!$doc_root || !$server_admin){
        print qq~<form method="post" action="$::ENV{'SCRIPT_NAME'}">
<input type="hidden" name="module" value="subdomains">
<input type="hidden" name="action" value="mod">
<input type="hidden" name="name" value="$subdomain">
<table border="0" cellpadding="1" cellspacing="0" align="center" width="400">
 <tr>
  <td colspan="2" align="center"><font face=\"verdana, arial\" size="+1"><b><u>Modify Subdomain</u></b></font></td>
 </tr>
 <tr>
  <td colspan="2" align="center"><font face=\"verdana, arial\" size=\"2\">Modifiying subdomain $virtualhost</font></td>
 </tr>
 <tr>
  <td><font face=\"verdana, arial\" size=\"2\">Document root:</font></td>
  <td><select name="DocumentRoot" class="inputbox">
~;

    chdir($::UserConf{'Home'});
    my $current_dir = '.';

    opendir(HOME, $current_dir);
    my @dirs = readdir(HOME);
    closedir HOME;
    @dirs = API::filterdirs($current_dir, @dirs);

    my @tmp_arr;
    foreach(@dirs)
    {
        $_ =~ s/^\.\///;
        $current_dir .= "/$_";
        opendir(SUBDIR, $current_dir);
        @tmp_arr = API::filterdirs($current_dir, readdir(SUBDIR));
        @dirs = (@dirs, @tmp_arr);
        closedir(SUBDIR);
        $current_dir = '.';
    }
    undef @tmp_arr;

    $doc_root =  $current{'DocumentRoot'};
    $doc_root =~ s/^$::UserConf{'Home'}//i;

    foreach(sort @dirs)
    {
        if($_ eq $doc_root){
            print "   <option value=\"$_\" selected>$_</option>\n";
        }else{
            print "   <option value=\"$_\">$_</option>\n";
        }
    }

    print qq~</td>
 </tr>
 <tr>
  <td><font face=\"verdana, arial\" size=\"2\">Contact email:</font></td>
  <td><input type="text" name="ServerAdmin" value="$current{'ServerAdmin'}" class="inputbox"></td>
 </tr>
 <tr>
  <td colspan="2" align="center"><input type="submit" value="Submit Changes"></td>
 </tr>
</table>
</form>~;
    return 1;
    }

    if(!-e "$::UserConf{'Home'}/$doc_root" || !-d "$::UserConf{'Home'}/$doc_root")
    {
        print "<font face=\"verdana, arial\" size=\"2\">Error: Document root not found!</font>\n";
        return 0;
    }

    my $update;
    my $changes_made = 0;
    unless($doc_root eq $current{'DocumentRoot'}){
        $update = $db->prepare("update httpd set DocumentRoot = '$::UserConf{'Home'}/$doc_root' where ServerName = '$virtualhost';");
        $update->execute;
        $changes_made++;
    }

    unless($server_admin eq $current{'ServerAdmin'}){
        $update = $db->prepare("update httpd set ContactEmail = '$server_admin' where ServerName = '$virtualhost';");
        $update->execute;
        $changes_made++;
    }
    undef $update;

    if($changes_made)
    {
        printf("<font face=\"verdana, arial\" size=\"2\">Subdomain %s successfully updated. %d change(s) were(was) made.</font>\n", $virtualhost, $changes_made);
        API::LogAction("$changes_made changes made to $virtualhost");
    }
    else
    {
        print "<font face=\"verdana, arial\" size=\"2\">No changes were made because the data you submitted is already current.</font>\n";
    }
}

# Make a new directory.
sub MakeDir
{
    my $dir = $_[0];

    if(!$dir) {
        print "<h1 align=\"center\"><u>Make New Directory</u></h1>\n";
        print "<p align=\"center\">\n";
        print "<form method=\"get\" action=\"index.cgi\">\n";
        print "<input type=\"hidden\" name=\"module\" value=\"subdomains\">\n";
        print "<input type=\"hidden\" name=\"action\" value=\"makedir\">\n";
        print "<input type=\"hidden\" name=\"oldaction\" value=\"$::HTTP_Args{'oldaction'}\">\n";
        print "<font face=\"verdana, arial\" size=\"2\">Name of folder to create: </font><input type=\"text\" name=\"name\" value=\"\" max=\"40\"><br>\n";
        print "<input type=\"submit\" value=\"Submit\">\n";
        print "</form>\n";
        print "</p>\n";
    }
    else {
        if($dir =~ /^\// || $dir =~ /\.\./) {
            print "<b><font face=\"verdana, arial\" size=\"2\" color=\"#FF0000\">Invalid directory. You may not use .. in the folder name, nor may it begin with a /.</font></b>\n";
        }
        else {
            if(mkdir("$::UserConf{'Home'}/$dir")) {
                chown((getpwnam($AUTH::User))[2, 3], "$::UserConf{'Home'}/$dir");
                print "<font face=\"verdana, arial\" size=\"2\">The directory $::UserConf{'Home'}/$dir was successfully created.</font>\n";
                API::LogAction("User $AUTH::User created directory $::UserConf{'Home'}/$dir");
            }
            else {
                print "<font face=\"verdana, arial\" size=\"2\">Failed to create directory. The operating system returned the following error: $!</font>\n";
            }
        }
    }
}

### Main Function ###
sub ::Main
{
    my($argc, %args) = @_;
    my $action = $::HTTP_Args{'action'};

    # Connect to the database before we do anything.
    $db = DBI->connect("DBI:mysql:$db_name:$db_host", $db_user, $db_pass) || print "<font face='verdana, arial' size='2'>The following error occured connecting to the database: $!</font>";

    # Find out how many subdomains this user currently has.
    $NumberOfSubs = CountSubs();

    # Do whatever it is they're trying to do *before* we display the list.
    if($action eq 'add')
    {
        if(AddSubdomain($::HTTP_Args{'name'}))
        {
            PrintBackLink();
            $db->disconnect;
            return;
        }
    }
    elsif($action eq 'del')
    {
        DeleteSubdomain($::HTTP_Args{'name'});
    }
    elsif($action eq 'mod')
    {
        if(ModifySubdomain($::HTTP_Args{'name'}))
        {
            PrintBackLink();
            $db->disconnect;
            return;
        }
    }
    elsif($action eq 'makedir')
    {
        MakeDir($::HTTP_Args{'name'});
        PrintBackLink();
        $db->disconnect;
        return;
    }

    my $sort = $HTTP_Args{'sort'};
    ($sort ne 'ServerName' || $sort ne 'DocumentRoot' || $sort ne 'ServerAdmin') and $sort = "ServerName";

    # List each sub-domain.
    my $list = $db->prepare("select * from httpd order by $sort");
    $list->execute;
    print "<table border=\"1\" cellpadding=\"2\" cellspacing=\"1\" align=\"center\" width=\"500\">\n";
    print " <tr>\n  <td align=\"center\" colspan=\"5\"><h3><font face=\"verdana, arial\"><u>Subdomains</u></font></h3></td>\n </tr>\n";
    print " <tr>\n";
    print "  <td><font face=\"verdana, arial\" size=\"2\"><u><a href=\"index.cgi?module=subdomains&sort=ServerName\" class=\"nohighlight\">Subdomain</a></u></font></td>\n";
    print "  <td><font face=\"verdana, arial\" size=\"2\"><u><a href=\"index.cgi?module=subdomains&sort=DocumentRoot\" class=\"nohighlight\">Document Root</a></u></font></td>\n";
    print "  <td><font face=\"verdana, arial\" size=\"2\"><u><a href=\"index.cgi?module=subdomains&sort=ServerAdmin\" class=\"nohighlight\">Contact E-mail</a></u></font></td>\n";
    print "  <td><font face=\"verdana, arial\" size=\"2\"><u>Modify</u></font></td>\n";
    print "  <td><font face=\"verdana, arial\" size=\"2\"><u>Delete</u></font></td>\n";
    print " </tr>\n";
    my $subdomain, @row;
    my $table_outputted = 0;
    SUBDOM: while(@row = $list->fetchrow_array)
    {
        if($row[2] eq $AUTH::User)
        {
            next SUBDOM if($row[5] eq $::UserConf{'Domain'});    # Skip the main virtualhost, that's not a subdomain.
            $subdomain = $row[0];
            $subdomain =~ s/\.$::UserConf{'Domain'}$//i;
            print " <tr>\n";
            print "  <td><font face=\"verdana, arial\" size=\"2\"><a href=\"http://$row[0]/\" target=\"_blank\" class=\"nohighlight\" style=\"text-decoration: none\">$row[0]</a></font></td>\n";
            print "  <td><font face=\"verdana, arial\" size=\"2\">$row[3]</font></td>\n";
            if($row[4]){
                print "  <td><font face=\"verdana, arial\" size=\"2\"><a href=\"mailto:$row[4]\" class=\"nohighlight\" style=\"text-decoration: none\">$row[4]</a></font></td>\n";
            }else{
                print "  <td align=\"center\"><font face=\"verdana, arial\" size=\"2\">-</font></td>\n";
            }
            print "  <td><a href=\"index.cgi?module=subdomains&action=mod&name=$subdomain\"><img src=\"/themes/$::Global_Config{'CurrentTheme'}[1]/button.gif\" border=\"0\"></a></td>\n";
            print "  <td><a href=\"index.cgi?module=subdomains&action=del&name=$subdomain\"><img src=\"/themes/$::Global_Config{'CurrentTheme'}[1]/button.gif\" border=\"0\"></a></td>\n";
            print " </tr>\n";
            $table_outputted = 1 if(!$table_outputted);
        }
    }
    undef $subdomain;
    if(!$table_outputted)
    {
        print " <tr><td colspan=\"5\" align=\"center\"><font face=\"verdana, arial\" size=\"2\">No subdomains were found.</font></td></tr>\n";
    }
    print "</table>\n";

    print "<p align=\"center\"><font face=\"verdana, arial\" size=\"2\"><a href=\"index.cgi?module=subdomains&action=add\">Add Subdomain</a> \| <a href=\"index.cgi?module=subdomains\">Refresh List</a></font></p>\n";

    $db->disconnect;
}
### End Main Function ###

# How many subdomains does this user have?
sub CountSubs
{
    my @row;
    my $count = 0;

    my $list = $db->prepare("select * from httpd");
    $list->execute;
    SUBDOM: while(@row = $list->fetchrow_array)
    {
        if($row[2] eq $AUTH::User)
        {
            next SUBDOM if($row[0] eq $::UserConf{'Domain'});    # Skip the main virtualhost, that's not a subdomain.
            $count++;
        }
    }

    return $count;
}

# Print a link at the bottom to make life a little easier.
sub PrintBackLink
{
        print "<p align=\"center\"><font face=\"verdana, arial\" size=\"2\"><a href=\"index.cgi?module=subdomains&action=$::HTTP_Args{'oldaction'}\">Back...</a></font></p>\n";
}

1;
